#ifndef COMMON_H
#define COMMON_H

#include <QProcess>
#include <QSettings>
#include <QFile>
#include <QDBusInterface>

namespace Common {

//加固解固代码
class ChattrDir
{
public:
    ChattrDir()
    {
        if(count == 0)
        {
            chattr = getChattrType();
            if(chattr && QFile::exists("/usr/bin/nde-chattr"))
            {
                QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.chattr"),
                                                             QString::fromLatin1("/ndeChattr"),
                                                             QString::fromLatin1("com.nde.chattr"),
                                                             QDBusConnection::systemBus());
                interface->call("cannelChattr");
                delete interface;
            }
        }
        count++;
    }
    ~ChattrDir()
    {
        count--;

        if(count == 0)
        {
            if( chattr && QFile::exists("/usr/bin/nde-chattr"))
            {
                QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.chattr"),
                                                             QString::fromLatin1("/ndeChattr"),
                                                             QString::fromLatin1("com.nde.chattr"),
                                                             QDBusConnection::systemBus());
                interface->call("addChattr");
                delete interface;
            }
        }
    }

    //获取加固状态
    static bool isChattr()
    {
        //若没有正在执行的加固解固动作，直接获取文件的加固解固保存的状态
        if(count == 0)
            return getChattrType();
        //否则返回保存的加固解固状态
        return chattr;
    }

    //判断加固命令是否存在
    static bool isChattrExist()
    {
        return QFile::exists("/usr/bin/nde-chattr");
    }

private:
    static bool getChattrType()
    {
        bool isChattr = false;
        if(QFile::exists("/usr/bin/nde-chattr"))
        {
            QSettings settings("/etc/nde/nde-chattr.conf", QSettings::IniFormat);
            settings.beginGroup("nde_chattr");
            isChattr = settings.value("currentChattr").toBool();
            settings.endGroup();
        }
        return isChattr;
    }

    static bool chattr;
    //计算该对象被创建的次数，用来判断是否多次执行了解固加固
    static int count;
};
inline int ChattrDir::count = 0;
inline bool ChattrDir::chattr = false;

bool checkVirtualMachine();
bool checkZteVersion();
inline bool checkVirtualMachine()
{
    QProcess p;
    QStringList options;

    //检测是否是虚拟机vbox vmware，通过读取scsi文件中的设备判断虚拟机
    //    options << "-c" << "cat /proc/scsi/scsi | grep -i -e vbox -e vmware -e virtualbox";
    //读取cpuinfo，cpuinfo中包含hypervisor字段说明是虚拟机
    options << "-c" << "grep -q \"^flags.*\\ hypervisor\\ \" /proc/cpuinfo";
    p.start("/bin/bash", options);
    p.waitForFinished();
    p.waitForReadyRead();

    //exitCode=0 说明命令执行成功，是虚拟机;exitCode=1表示命令执行失败，是实体机
    int exitCode = p.exitCode();
    return exitCode ? false:true;
}

inline bool checkZteVersion()
{
    QSettings setting("/etc/os-release", QSettings::IniFormat);
    QString version = setting.value("VERSION").toString();
    QString versionId = setting.value("VERSION_ID").toString();
    return (version.contains("zte", Qt::CaseInsensitive) || versionId.contains("zte", Qt::CaseInsensitive));
}
};

#endif // COMMON_H
