#include "messagewidget.h"

#include <QWidget>
#include <QHBoxLayout>
#include <QPoint>
#include <QPainter>
#include <QImage>
#include <QVariant>
#include <QGraphicsDropShadowEffect>
#include <QPainterPath>

MessageWidget::MessageWidget(QWidget *parent):
    QWidget(parent),
    m_offset(50),
    m_triangleWidth(TRIANGLE_WIDTH),
    m_triangleHeight(TRIANGLE_HEIGHT)
{

    setWindowFlags(Qt::FramelessWindowHint);
    setAttribute(Qt::WA_TranslucentBackground);

    // 设置阴影边框
    auto shadowEffect = new QGraphicsDropShadowEffect(this);
    shadowEffect->setOffset(0, 0);
    shadowEffect->setColor(QColor("#d8d8d8"));
    shadowEffect->setBlurRadius(SHADOW_WIDTH);
    this->setGraphicsEffect(shadowEffect);

    QHBoxLayout* hMainLayout = new QHBoxLayout;
    setLayout(hMainLayout);

    m_labeltext = new QLabel();
    m_labeltext->setProperty("class", "font14px");

    m_labeltext->setWordWrap(true);
    m_labeltext->setAlignment(Qt::AlignLeft);
    m_labeltext->setMidLineWidth(10);
    m_labeltext->setMinimumHeight(10);
    m_labeltext->setStyleSheet(QLatin1String("QLabel { padding: 10px 20px 10px 14px; color: #ff3535;}"));
    hMainLayout->addWidget(m_labeltext);
    this->setStyleSheet(QLatin1String("background-color: #ffffff; "));
}

MessageWidget::~MessageWidget() {

}

// 设置小三角显示的起始位置;
void MessageWidget::setStartPos(int startX)
{
    m_offset = startX;
    repaint();
}

void MessageWidget::setTriangleInfo(int width, int height)
{
    m_triangleWidth = width;
    m_triangleHeight = height;
}


void MessageWidget::setText(QString s)
{
    m_labeltext->setText(s);
    adjustSize();
}


void MessageWidget::setDirection(MessageWidget::Direction d)
{
    m_direct = d;
}


QString MessageWidget::text()
{
    return m_labeltext->text();
}

void MessageWidget::triangleMove(int x, int y)
{
    int top_left_x, top_left_y;
    switch (m_direct) {
    case down:
        top_left_x = x - m_offset - m_triangleWidth / 2 - m_labeltext->x();
        top_left_y = y - m_triangleHeight - m_labeltext->height() - m_labeltext->y();
        move(QPoint(top_left_x, top_left_y));
        break;
    case up:
        top_left_x = x - m_offset - m_triangleWidth / 2 - m_labeltext->x();
        top_left_y = y + m_triangleHeight - m_labeltext->y();
        move(QPoint(top_left_x, top_left_y));
        break;
    case left:
        top_left_x = x + m_triangleHeight - m_labeltext->x();
        top_left_y = y - m_offset - m_triangleWidth / 2 - m_labeltext->y();
        move(QPoint(top_left_x, top_left_y));
        break;
    case right:
        top_left_x = x - m_triangleHeight - m_labeltext->width() - m_labeltext->x();
        top_left_y = y - m_triangleWidth / 2 - m_offset - m_labeltext->y();
        move(QPoint(top_left_x, top_left_y));
        break;
    default:
        break;
    }

}


void MessageWidget::paintEvent(QPaintEvent*)
{

    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(Qt::NoPen);
    painter.setBrush(QColor(255, 255, 255));

    QPainterPath drawPath;
    // 小三角区域;
    QPolygon trianglePolygon;

    QRect wRect(m_labeltext->x(), m_labeltext->y(), m_labeltext->width(), m_labeltext->height());

    // 设置小三的具体位置
    int tri_pos_x, tri_pos_y;
    switch (m_direct)
    {
    case up:{
        // 小三角左边的点的位置
        tri_pos_x = wRect.x() + m_offset;
        tri_pos_y = wRect.y();
        trianglePolygon << QPoint(tri_pos_x, tri_pos_y);
        trianglePolygon << QPoint(tri_pos_x + m_triangleWidth, tri_pos_y);
        trianglePolygon << QPoint(tri_pos_x + m_triangleWidth / 2, tri_pos_y - m_triangleHeight);
    }
        break;
    case left:{
        // 小三上边点的位置
        tri_pos_x = wRect.x();
        tri_pos_y = wRect.y() + m_offset;
        trianglePolygon << QPoint(tri_pos_x, tri_pos_y);
        trianglePolygon << QPoint(tri_pos_x - m_triangleHeight, tri_pos_y + m_triangleWidth / 2);
        trianglePolygon << QPoint(tri_pos_x, tri_pos_y + m_triangleWidth);
    }
        break;
    case right:{
        // 小三上边点的位置
        tri_pos_x = wRect.x() + wRect.width();
        tri_pos_y = wRect.y() + m_offset;
        trianglePolygon << QPoint(tri_pos_x, tri_pos_y);
        trianglePolygon << QPoint(tri_pos_x + m_triangleHeight, tri_pos_y + m_triangleWidth / 2);
        trianglePolygon << QPoint(tri_pos_x, tri_pos_y + m_triangleWidth);
    }

        break;
    case down:{
        // 小三左边点的位置
        tri_pos_x = wRect.x() + m_offset;
        tri_pos_y = wRect.y() + wRect.height();
        trianglePolygon << QPoint(tri_pos_x, tri_pos_y);
        trianglePolygon << QPoint(tri_pos_x + m_triangleWidth / 2, tri_pos_y + m_triangleHeight);
        trianglePolygon << QPoint(tri_pos_x + m_triangleWidth, tri_pos_y);
    }
        break;
    default:
        break;
    }
    drawPath.addRoundedRect(wRect, BORDER_RADIUS, BORDER_RADIUS);
    drawPath.addPolygon(trianglePolygon);
    painter.drawPath(drawPath);
}


void MessageWidget::mousePressEvent(QMouseEvent *)
{
    this->close ();
}

// 判断是否为纯数字
bool MessageWidget::isDigitStr(QString src) {
    bool isDigit;
    bool isDouble;
    src.toInt(&isDigit);
    src.toDouble(&isDouble);
    return isDigit || isDouble;
}


void MessageWidget::showWidget(int x, int y, QString message) {
    triangleMove(x, y);
    setText(message);
    show();
}
