#ifndef MESSAGEWIDGET_H
#define MESSAGEWIDGET_H

#include <QWidget>
#include <QGraphicsDropShadowEffect>
#include <QLabel>

#define PORTABLE_FS_CHARS "[^a-zA-Z0-9._-]"
#define PORTABLE_END_CHARS "[^a-zA-Z0-9]"
#define MAXINPUTSIZE 16

const int SHADOW_WIDTH = 10;                 // 窗口阴影宽度;
const int TRIANGLE_WIDTH = 15;               // 小三角的宽度;
const int TRIANGLE_HEIGHT = 8;              // 小三角的高度;
const int BORDER_RADIUS = 4;                 // 窗口边角的弧度;

class MessageWidget : public QWidget
{
    Q_OBJECT
public:
    MessageWidget(QWidget* parent = nullptr);
    ~MessageWidget();

    enum Direction{
        left,
        right,
        up,
        down
    };
    // 设置小三角起始位置;
    void setStartPos(int startX);
    // 设置小三角宽和高;
    void setTriangleInfo(int width, int height);
    // 设置展示文本
    void setText(QString s);
    // 设置小三角的位置
    void setDirection(Direction d);

    QString text();
    // 比起左上角的位置  用户更关心小三角的尖尖的位置 重载move以便用户更容易定位气泡框的位置
    // x,y 是气泡窗口小尖尖的坐标
    void triangleMove(int x, int y);

    bool isDigitStr(QString src);

    void showWidget(int x, int y, QString text);

protected:
    void paintEvent(QPaintEvent *);
    void mousePressEvent (QMouseEvent *);

private:
    // 小三角的偏移量;
    int m_offset;
    // 小三角的宽度;
    int m_triangleWidth;
    // 小三角高度;
    int m_triangleHeight;
    // 小三角方向;
    Direction m_direct;
    QLabel *m_labeltext;



};

#endif // MESSAGEWIDGET_H
