#ifndef STACKPAGE_H
#define STACKPAGE_H

#include <QWidget>
#include <QToolButton>
#include <QList>
#include <Nde/GridLayout>
#include <QStack>
#include "page.h"
#include <QTimer>
#include <QMap>

namespace Ui {
class StackPage;
}

class StackPage : public QWidget
{
    Q_OBJECT
public:
    explicit StackPage(QWidget *parent = nullptr);
    ~StackPage();

    //每次进入该界面需要刷新当前页面
    void refreshStackPage();

protected:
    //创建页面，要在子类中实现
    virtual Page * createPage(const QString key) = 0;
    //获取key页面
    Page * getPage(const QString key);
    //获取TOP页面
    Page * getTopPage();
    //设置页面堆栈首页，下一页由信号触发
    void setFirstPage(const QString key);
    //设置下一页
    void setNextPage(const QString key);
    //切换页面
    void setSwitchPage(const QString key);

private Q_SLOTS:
    //页面发送返回请求
    void slotBackPage();
    //OnePage内触发切换到下一页
    void slotNextPage(const QString key);
    //OnePage内触发切换到另一页
    void slotSwitchPage(const QString key);

    //标题发生变化
    void slotPageTitleChanged(QString text);
    //OnePage底部按钮是否可见，可点击
    void slotButtomBtnVisibleChanged(int key, bool visible);

private Q_SLOTS:
    void slotBack();
    void slotBottom();
    void slotScrollAreaValueChanged(int value);

private:
    //添加一页
    void addTopPage(Page *page);
    //删除当前页
    void removeTopPage();
    //根据新页面刷新显示，删除，添加页面后的处理
    void updateTopPage();

private:
    //设置页面标题
    void setPageTitle();

    //设置返回键是否显示,是否禁用
    void setBackBtnVisible();

    //添加底端按钮
    void addBottomBtns();
    void setBottomBtnsVisible();
    //清除低端所有按钮
    void clearBottomBtns();

private:
    Ui::StackPage *ui;

private:
    //页面的栈信息，处理页面返回
    QStack<Page*> _stackPage;
    //<OnePage::pageKey(), OnePage> 所有页面，需要切换到具体页面时可以动态添加
    QMap<QString, Page*> _pages;

private:
    QTimer * _effectTimer{nullptr};
    QHBoxLayout * _bottomLayout{nullptr};
    QList<QToolButton*> _bottomBtns;
};

#endif // STACKPAGE_H
