#include "cabledetail.h"
#include "ui_cabledetail.h"
#include "netinterface.h"
#include "../baseui/common.h"
#include <NetworkManagerQt/Manager>
#include "../baseui/baseuitr.h"
#include <QThread>

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_CONNECT (Page::PAGE_BBTN_USER+1)
#define PAGE_BUTTOM_BTN_DISCONNECT (Page::PAGE_BBTN_USER+2)

CableDetail::CableDetail(QWidget *parent) :
    Page(parent),
    ui(new Ui::CableDetail)
{
    ui->setupUi(this);

    QFile f(":/qss/cabledetail.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
}

CableDetail::~CableDetail()
{
    delete ui;
}

void CableDetail::setWiredInfo(NetworkManager::Device::Ptr d,NetworkManager::Connection::Ptr conn)
{
    device = d;
    m_conn = conn;
    onactiveConnectionsChanged();
    connect(device.data(), &NetworkManager::Device::stateChanged, this, &CableDetail::onactiveConnectionsChanged);

    initui();

    if(detail)
        detail->setWired(device,conn);
    if(ipv4){
        ipv4->setdevice(device.data()->interfaceName(),conn);
    }
    if(ipv6){
        ipv6->setdevice(device.data()->interfaceName(),conn);
    }
    ui->linename->setText(conn->name());
}

void CableDetail::applyBtnClicked()
{
    Common::ChattrDir chat;
    NetworkManager::ConnectionSettings::Ptr connectionSettings = m_conn->settings();
    QString name = ui->linename->text();
    connectionSettings.data()->setId(name);
    QDBusPendingReply<> reply = m_conn->update(connectionSettings->toMap());
    reply.waitForFinished();
    if(!reply.isValid())
        qDebug() << "============" << reply.error();
    m_conn->save();

    if(ipv4){
        if(!ipv4->save()){
            return;
        }
    }
    if(ipv6){
        if(!ipv6->save()){
            return ;
        }
    }
}

QList<QPair<int, QString> > CableDetail::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_CONNECT, tr("Connect"));
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_DISCONNECT, tr("DisConnect"));
    pairs << QPair<int, QString>(Page::PAGE_BBTN_APPLY, g_tr_apply);
    return pairs;
}

QList<int> CableDetail::buttomBtnsVisible()
{
    QList<int> page_bbtn_keys;
    page_bbtn_keys << Page::PAGE_BBTN_APPLY;
    return page_bbtn_keys;
}

void CableDetail::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_CONNECT)
    {
        Connect();
    }
    if(key == PAGE_BUTTOM_BTN_DISCONNECT)
    {
        Disconnect();
    }
}

void CableDetail::refreshPage()
{
//    if(checkactive()){
//        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DISCONNECT, true);
//        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_CONNECT, false);
//    } else {
//        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DISCONNECT, false);
//        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_CONNECT, true);
//    }
}

void CableDetail::onactiveConnectionsChanged()
{
    QThread::msleep(100);
    if(detail)
        detail->setWired(device,m_conn);
    if(ipv4){
        ipv4->setdevice(device.data()->interfaceName(),m_conn);
    }
    if(ipv6){
        ipv6->setdevice(device.data()->interfaceName(),m_conn);
    }
    if(checkactive()){
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DISCONNECT, true);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_CONNECT, false);
    } else {
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DISCONNECT, false);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_CONNECT, true);
    }
}

bool CableDetail::checkactive()
{
    //获取可用的有线连接
    for (auto const & conn : NetworkManager::activeConnections())
    {
        if((conn->connection() == m_conn) && (conn->state() == NetworkManager::ActiveConnection::Activated))
        {
            return true;
        }
    }
    return  false;
}

void CableDetail::initui()
{
    if(!detail){
        detail = new DetailInfo(this);
        ui->lay_detail->addWidget(detail);
    }

    QString interfacename = device.data()->interfaceName();

    if(!ipv4){
        ipv4 = new IPSetting(this,device.data()->interfaceName(),true);
        ui->lay_v4->addWidget(ipv4);
    }
    if(!ipv6){
        ipv6 = new IPSetting(this,device.data()->interfaceName(),false);
        ui->lay_v6->addWidget(ipv6);
    }
}

void CableDetail::Connect()
{
    QString spec_object;

    QString conn_uni = m_conn->path();
    QString conn_name = m_conn->name();

    QString dev_name = device->interfaceName();
    QString dev_uni = device->uni();
    if(dev_uni.isEmpty())
    {
        qWarning() << QStringLiteral("找不到设备['%1']去激活有线连接['%2']。").arg(dev_name).arg(conn_name);
        return ;
    }

    QDBusPendingCallWatcher * watcher;
    watcher = new QDBusPendingCallWatcher{NetworkManager::activateConnection(conn_uni, dev_uni, spec_object), this};
    connect(watcher, &QDBusPendingCallWatcher::finished, [conn_name, dev_name] (QDBusPendingCallWatcher * watcher) {
        if (watcher->isError() || !watcher->isValid())
        {
            qWarning() << QStringLiteral("激活设备['%1']的有线连接['%2']失败: %3。").arg(dev_name).arg(conn_name).arg(watcher->error().message());
        }
        watcher->deleteLater();
    });
}

void CableDetail::Disconnect()
{
    NetworkManager::ActiveConnection::Ptr aConn;
    NetworkManager::ActiveConnection::List aConnList = NetworkManager::activeConnections();
    for(NetworkManager::ActiveConnection::Ptr p: aConnList)
    {
        if(m_conn->uuid() == p->uuid())
        {
            aConn = p;
            break;
        }
    }
    if(aConn && aConn->isValid())
    {
        QDBusPendingReply<> reply = NetworkManager::deactivateConnection(aConn->path());
        QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
        connect(watcher, &QDBusPendingCallWatcher::finished, [aConn] (QDBusPendingCallWatcher * watcher) {
            if (watcher->isError() || !watcher->isValid())
            {
                qWarning() << QStringLiteral("取消有线连接['%1']失败: %2。").arg(aConn->id()).arg(watcher->error().message());
            }
            watcher->deleteLater();
        });
    }
}
