#include "cableitem.h"
#include "ui_cableitem.h"
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include <NetworkManagerQt/WirelessSetting>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/Ipv4Setting>
#include <NetworkManagerQt/Ipv6Setting>
#include <NetworkManagerQt/Utils>
#include <NetworkManagerQt/WiredDevice>
#include <QTimer>
#include <QFile>
CableItem::CableItem(QWidget *parent) :
    QFrame(parent),
    ui(new Ui::CableItem)
{
    ui->setupUi(this);
    QFile f(":/qss/cableitem.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    connect(ui->btn_set, &QPushButton::clicked, this, &CableItem::onSetClicked);

}

CableItem::~CableItem()
{
    delete ui;
}

void CableItem::onSetClicked()
{
    Q_EMIT DetailInfo(this);
}
bool CableItem::checkactive()
{
     //获取可用的有线连接
    for (auto const & conn : NetworkManager::activeConnections())
    {
        if((conn->connection() == m_conn) && (conn->state() == NetworkManager::ActiveConnection::Activated))
        {
            return true;
        }
    }


    return  false;
}
void CableItem::onactiveConnectionsChanged()
{
    QTimer::singleShot(1000,this,[this](){
        ui->label_con->setVisible(checkactive());
        ui->cablename->setText(m_conn.data()->name());
        Q_EMIT ConnectedChanged();
    });

}

void CableItem::setDetailInfo(NetDetailInfo *tmp)
{
    info = tmp;
    ui->label_con->setVisible(false);
    ui->cablename->setText(info->getNetName());
}

void CableItem::setWired(NetworkManager::Device::Ptr d,NetworkManager::Connection::Ptr conn)
{
    device = d;
    m_conn = conn;
    ui->label_con->setVisible(false);
    QString name = device.data()->interfaceName();
    QString uni = device->uni();
    QString uuid = device.data()->udi();
    ui->cablename->setText(conn.data()->name());
    connect(device.data(), &NetworkManager::Device::activeConnectionChanged, this, &CableItem::onactiveConnectionsChanged);
    connect(m_conn.data(), &NetworkManager::Connection::updated,this, &CableItem::onConnectUpdated);
    QTimer::singleShot(200,this,[this](){
        ui->label_con->setVisible(checkactive());
        ui->cablename->setText(m_conn.data()->name());
    });

}

void CableItem::onConnectUpdated()
{
    QTimer::singleShot(1000,this,[this](){
        ui->label_con->setVisible(checkactive());
        ui->cablename->setText(m_conn.data()->name());
    });
}

NetDetailInfo * CableItem::getDetailInfo()
{
    return info;
}

NetworkManager::Device::Ptr CableItem::getWired()
{
    return device;
}
NetworkManager::Connection::Ptr CableItem::getConn()
{
    return m_conn;
}
