#include "cablenetwork.h"
#include "ui_cablenetwork.h"
#include "cableitem.h"
#include <QProcess>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Device>
#include <NetworkManagerQt/ConnectionSettings>
#include <NetworkManagerQt/Settings>
#include <QFile>

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_ADD (Page::PAGE_BBTN_USER+1)

CableNetwork::CableNetwork(QWidget *parent) :
    Page(parent),
    ui(new Ui::CableNetwork)
{
    ui->setupUi(this);

    QFile f(":/qss/cablenetwork.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    QHBoxLayout * lay_off = new QHBoxLayout();
    lay_off->setContentsMargins(0,0,0,0);
    lay_off->setSpacing(0);
    ui->widget->setLayout(lay_off);
    toggle_off = new NdeWidget::QMaterialToggle(ui->widget);
    toggle_off->setChecked(enable());
    connect(toggle_off, &NdeWidget::QMaterialToggle::toggled, this, &CableNetwork::onSwitchOff);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::networkingEnabledChanged, this, &CableNetwork::onnetworkingEnabledChanged);

    lay_off->addWidget(toggle_off);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::deviceAdded, this, &CableNetwork::onDeviceAdded);
    initui();
}

CableNetwork::~CableNetwork()
{
    delete ui;
}

QList<QPair<int, QString> > CableNetwork::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_ADD, tr("Add"));
    return pairs;
}

QList<int> CableNetwork::buttomBtnsVisible()
{
//    QList<int> page_bbtn_keys;
//    page_bbtn_keys << PAGE_BUTTOM_BTN_ADD;
    return QList<int>();
}

void CableNetwork::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_ADD)
    {
        Q_EMIT nextPage(CableAdd::pageKey());
    }
}

void CableNetwork::refreshPage()
{
    //
}

void CableNetwork::onConnectionRemoved(const QString &path)
{
    for(CableItem * item : itemlist)
    {
        qWarning() << "删除连接----" << item->getConn()->path() << "   " << path;
        if(item->getConn()->path() == path)
        {
            qWarning() << "删除连接" << item->getConn()->name();
            itemlist.removeAll(item);
            ui->lay_item->removeWidget(item);
            if(mConnlist.contains(item->getConn())){
                mConnlist.removeAll(item->getConn());
            }
            delete item;
            break;
        }
    }
}


void CableNetwork::onDeviceAdded(const QString &uni)
{
    //新建有线连接
    NetworkManager::Connection::Ptr addConn = NetworkManager::findConnection(uni);

    if(mConnlist.contains(addConn)){
        return;
    }
    //每次删除连接时，
    if(addConn && addConn->isValid() && !addConn->name().isEmpty())
    {
        //查找该连接所在的设备
        NetworkManager::Device::Ptr wiredDevice;
        for(NetworkManager::Device::Ptr device : mDevices)
        {
            NetworkManager::WiredDevice * d = qobject_cast<NetworkManager::WiredDevice *>(device.data());
            if(d)
            {
                for (NetworkManager::Connection::Ptr const & dev_conn : d->availableConnections())
                {
                    qWarning() << dev_conn->path() << "-----" << addConn->path();
                    if (dev_conn == addConn)
                    {
                        wiredDevice = device;
                        break;
                    }
                }
            }
            if(wiredDevice) break;
        }
        if(wiredDevice)
        {
            qWarning() << "新建有线连接" << addConn->name();
            CableItem * item1 = new CableItem(this);
            item1->setWired(wiredDevice,addConn);
            mConnlist.append(addConn);
            connect(item1,&CableItem::DetailInfo,this,&CableNetwork::onDetailInfo);
            connect(item1,&CableItem::ConnectedChanged,this,&CableNetwork::onConnectedChanged);
            ui->lay_item->addWidget(item1);
            itemlist.push_back(item1);
        }
    }
}
void CableNetwork::onnetworkingEnabledChanged(bool enbled)
{
    if(enbled){
        initui();
    } else {
        clearui();
    }
}
void CableNetwork::initui()
{
    for (auto const & device : NetworkManager::networkInterfaces())
    {
        //该对象只获取有线和无线设备

        qWarning() << "DEVICE INFORMATION:" << device->interfaceName() << " "<< device->driver() <<" "<< device->driverVersion() << device->type();
        if(device->type() == NetworkManager::Device::Ethernet )
        {
            if(!mDevices.contains(device)){
                mDevices.append(device);
            }
            NetworkManager::WiredDevice * wiredDevice = qobject_cast<NetworkManager::WiredDevice *>(device.data());
            connect(device.data(), &NetworkManager::Device::availableConnectionAppeared, this, &CableNetwork::onDeviceAdded);
            connect(device.data(), &NetworkManager::Device::availableConnectionDisappeared, this, &CableNetwork::onConnectionRemoved);

            if(wiredDevice){
                for (auto conn : wiredDevice->availableConnections())
                {

                    if(mConnlist.contains(conn)){
                        continue;
                    }
                    CableItem * item1 = new CableItem(this);
                    item1->setWired(device,conn);
                    mConnlist.append(conn);
                    connect(item1,&CableItem::DetailInfo,this,&CableNetwork::onDetailInfo);
                    connect(item1,&CableItem::ConnectedChanged,this,&CableNetwork::onConnectedChanged);
                    ui->lay_item->addWidget(item1);
                    itemlist.push_back(item1);
                }
            }

       }

    }
}
void CableNetwork::clearui()
{
    for (int i = 0; i<itemlist.size();i++) {
        CableItem * info = itemlist.at(i);
        ui->lay_item->removeWidget(info);
        delete info;
        info = nullptr;
    }
    itemlist.clear();
    mConnlist.clear();
}
void CableNetwork::Connect(NetworkManager::Device::Ptr device,NetworkManager::Connection::Ptr m_conn)
{
    QString spec_object;

            //
    QString conn_uni = m_conn->path();
    QString conn_name = m_conn->name();

    QString dev_name = device->interfaceName();
    QString dev_uni = device->uni();
    if(dev_uni.isEmpty())
    {
        qWarning() << QStringLiteral("找不到设备['%1']去激活有线连接['%2']。").arg(dev_name).arg(conn_name);
        return ;
    }

    QDBusPendingCallWatcher * watcher;
    watcher = new QDBusPendingCallWatcher{NetworkManager::activateConnection(conn_uni, dev_uni, spec_object), this};
    connect(watcher, &QDBusPendingCallWatcher::finished, [conn_name, dev_name] (QDBusPendingCallWatcher * watcher) {
        if (watcher->isError() || !watcher->isValid())
        {
            qWarning() << QStringLiteral("激活设备['%1']的有线连接['%2']失败: %3。").arg(dev_name).arg(conn_name).arg(watcher->error().message());
        }
        watcher->deleteLater();
    });
}
void CableNetwork::Disconnect(NetworkManager::Device::Ptr device,NetworkManager::Connection::Ptr m_conn)
{
    NetworkManager::ActiveConnection::Ptr aConn;
    NetworkManager::ActiveConnection::List aConnList = NetworkManager::activeConnections();
    for(NetworkManager::ActiveConnection::Ptr p: aConnList)
    {
        if(m_conn->uuid() == p->uuid())
        {
            aConn = p;
            break;
        }
    }
    if(aConn && aConn->isValid())
    {
        QDBusPendingReply<> reply = NetworkManager::deactivateConnection(aConn->path());
        QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
        connect(watcher, &QDBusPendingCallWatcher::finished, [aConn] (QDBusPendingCallWatcher * watcher) {
            if (watcher->isError() || !watcher->isValid())
            {
                qWarning() << QStringLiteral("取消有线连接['%1']失败: %2。").arg(aConn->id()).arg(watcher->error().message());
            }
            watcher->deleteLater();
        });
    }
}

void CableNetwork::onSwitchOff()
{
    if(!toggle_off->isChecked()){
        for (auto const & device : NetworkManager::networkInterfaces())
        {
            if(device->type() == NetworkManager::Device::Ethernet )
            {
                device.data()->setAutoconnect(false);
                NetworkManager::WiredDevice * wiredDevice = qobject_cast<NetworkManager::WiredDevice *>(device.data());
                if(wiredDevice){
                    for (auto conn : wiredDevice->availableConnections())
                    {
                        Disconnect(device,conn);
                    }
                }
            }
        }
    } else {
        for (auto const & device : NetworkManager::networkInterfaces())
        {
            if(device->type() == NetworkManager::Device::Ethernet )
            {
                device.data()->setAutoconnect(true);
                NetworkManager::WiredDevice * wiredDevice = qobject_cast<NetworkManager::WiredDevice *>(device.data());
                if(wiredDevice){
                    for (auto conn : wiredDevice->availableConnections())
                    {
                        Connect(device,conn);
                    }
                }
            }
        }
    }
}

void CableNetwork::onConnectedChanged()
{
    toggle_off->setChecked(enable());
}

bool CableNetwork::enable()
{
    for (auto const & conn : NetworkManager::activeConnections())
    {
        if(conn->state() == NetworkManager::ActiveConnection::Activated)
        {
            return true;
        }
    }
    for (auto const & device : NetworkManager::networkInterfaces())
    {
        if(device->type() == NetworkManager::Device::Ethernet )
        {
            if(device.data()->autoconnect())
            {
                return true;
            }
        }
    }
    return false;
}

void CableNetwork::showEvent(QShowEvent *event)
{
    toggle_off->setChecked(enable());
    clearui();
    initui();
}

void CableNetwork::onDetailInfo(CableItem* item)
{
    Q_EMIT signalCableDetail(item->getWired(),item->getConn());
}

void CableNetwork::setitemVisible(bool flag)
{
    CableItem * item;
    for (int i = 0; i<itemlist.size();i++) {
        item = itemlist.at(i);
        item->setVisible(flag);
    }
}
