#include "detailinfo.h"
#include "ui_detailinfo.h"
#include <NetworkManagerQt/WirelessNetwork>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include <NetworkManagerQt/WirelessSetting>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/Ipv4Setting>
#include <NetworkManagerQt/Ipv6Setting>
#include <NetworkManagerQt/Utils>
#include <NetworkManagerQt/WiredDevice>
#include <QDBusInterface>

DetailInfo::DetailInfo(QWidget *parent) :
    QFrame(parent),
    ui(new Ui::DetailInfo)
{
    ui->setupUi(this);
    ui->line->setStyleSheet("border-top:1px solid #e5e5e7;");
    ui->line_2->setStyleSheet("border-top:1px solid #e5e5e7;");
    ui->line_3->setStyleSheet("border-top:1px solid #e5e5e7;");
//    ui->line_4->setStyleSheet("border-top:1px solid #e5e5e7;");
//    ui->line_5->setStyleSheet("border-top:1px solid #e5e5e7;");
}

DetailInfo::~DetailInfo()
{
    delete ui;
}

bool DetailInfo::checkactive()
{
    for (auto const & conn : NetworkManager::activeConnections())
    {
        if((conn->connection() == m_conn) && (conn->state() == NetworkManager::ActiveConnection::Activated))
        {
            return true;
        }
    }
    return  false;
}

void DetailInfo::setWired(NetworkManager::Device::Ptr device,NetworkManager::Connection::Ptr conn)
{
    m_conn = conn;
    ui->ipv4->setText("");
    ui->ipv6->setText("");
    ui->speed->setText("—");
    ui->mac->setText("");
    //获取网卡速度
    QString speedpath = device.data()->uni();
    QDBusInterface * _interface =  new QDBusInterface(QString::fromLatin1("org.freedesktop.NetworkManager"),
                                                    speedpath,
                                                    QString::fromLatin1("org.freedesktop.NetworkManager.Device.Wired"),
                                                    QDBusConnection::systemBus());
    if(_interface->isValid())
    {
        QVariant v = _interface->property("Speed");
        if(!v.isValid()){
            return;
        }
        int speed = v.toInt();
        if(speed == 0)
        {
            ui->speed->setText("—");
            ui->speed_unit->hide();
        }
        else
        {
            ui->speed->setText(QString::number(speed));
            ui->speed_unit->show();
        }
        QVariant mac = _interface->property("HwAddress");
        if(!mac.isValid()){
            return;
        }
        QString strmac = mac.toString();
        ui->mac->setText(strmac);
    }


    if(checkactive()){
        NetworkManager::ConnectionSettings::Ptr connectionSettings = m_conn->settings();
        NetworkManager::Ipv4Setting::Ptr ipv4Setting = connectionSettings->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();
        NetworkManager::Ipv6Setting::Ptr ipv6Setting = connectionSettings->setting(NetworkManager::Setting::Ipv6).dynamicCast<NetworkManager::Ipv6Setting>();

        if(ipv4Setting->method() == NetworkManager::Ipv4Setting::Automatic){
            NetworkManager::Dhcp4Config::Ptr dhcp4 = device->dhcp4Config();
            if(!dhcp4.isNull())
            {
                ui->ipv4->setText(dhcp4->optionValue("ip_address"));
            }
        } else {
            auto addr4 = ipv4Setting->addresses();
            if(!addr4.empty())
                ui->ipv4->setText(addr4.first().ip().toString());
        }


    if(ipv6Setting->method() == NetworkManager::Ipv6Setting::Automatic){
        NetworkManager::IpConfig ip6Config = device->ipV6Config();
        if (ip6Config.isValid()) {
            NetworkManager::IpAddress address = ip6Config.addresses().at(0);
            ui->ipv6->setText(address.ip().toString());
        }
    } else {
        auto addr6 = ipv6Setting->addresses();
        if(!addr6.empty())
            ui->ipv6->setText(addr6.first().ip().toString());
    }

    }
    return;

}

void DetailInfo::setDetailInfo(NetDetailInfo *info)
{
//    ui->speed->setText("1000");
    auto interlist = QNetworkInterface::allInterfaces();
    for (int i = 0; i < interlist.size(); i++) {
        auto interface = interlist.at(i);
        if(interface.type() == QNetworkInterface::Ethernet){
            QString netname = interface.humanReadableName();
            if(info->getNetName().compare(netname)==0){
                auto address = interface.allAddresses();
                for (int j = 0; j < address.size(); j++) {
                    auto item = address.at(j);
                    if(item != QHostAddress::LocalHost && item.toIPv4Address()){
                        QString ip =item.toString();
                        ui->ipv4->setText(ip);
                    }
                    if(item != QHostAddress::LocalHost && item.protocol()==QAbstractSocket::IPv6Protocol && (!item.isLoopback())){
                        QString ip =item.toString();
                        ip = ip.mid(0,ip.indexOf("%"));
                        ui->ipv6->setText(ip);
                    }
                }
                auto entry = interface.addressEntries();
                for (int j = 0; j < entry.size(); j++) {
                    auto item = entry.at(j);
                    info->setMask(item.netmask().toString());
                    info->setGateWay(item.broadcast().toString());
                }
            }
        }
    }
    ui->mac->setText(info->getMac());

    for (auto const & tmp : NetworkManager::networkInterfaces())
    {
        if(tmp->interfaceName() == info->getNetName())
        {
            //获取网卡速度
            QString path = tmp->uni();
            QDBusInterface * _interface =  new QDBusInterface(QString::fromLatin1("org.freedesktop.NetworkManager"),
                                                            path,
                                                            QString::fromLatin1("org.freedesktop.NetworkManager.Device.Wired"),
                                                            QDBusConnection::systemBus());
            if(_interface->isValid())
            {
                QVariant v = _interface->property("Speed");
                if(!v.isValid()){
                    return;
                }
                int speed = v.toInt();
                if(speed == 0)
                {
                    ui->speed->setText("—");
                    ui->speed_unit->hide();
                }
                else
                {
                    ui->speed->setText(QString::number(speed));
                    ui->speed_unit->show();
                }
            }
            return;
        }
    }


}
