#include "ipsetting.h"
#include "ui_ipsetting.h"
#include <NetworkManagerQt/WirelessNetwork>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include <NetworkManagerQt/WirelessSetting>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/Ipv4Setting>
#include <NetworkManagerQt/Ipv6Setting>
#include <NetworkManagerQt/Utils>
#include <NetworkManagerQt/WiredDevice>
#include <QDebug>
#include <QMessageBox>
#include "netinterface.h"
#include <QValidator>
#include <QFile>
#include <QProcess>


IPSetting::IPSetting(QWidget *parent,QString name,bool ipv4) :
    QWidget(parent),
    ui(new Ui::IPSetting),
    intername(name),
    bipv4(ipv4)
{
    ui->setupUi(this);

    QFile f(":/qss/ipsetting.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    if(bipv4){
        ui->ip_address->setVisible(false);
    }


    ui->combox_method->addItem(tr("Auto"));
    ui->combox_method->addItem(tr("Manual"));
    ui->combox_method->addItem(tr("Disable"));

    connect(ui->combox_method, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &IPSetting::onMethodIndexChanged);
    onMethodIndexChanged(0);
}

IPSetting::~IPSetting()
{
    delete ui;
}
void IPSetting::setdevice(QString name,NetworkManager::Connection::Ptr conn)
{
    intername = name;
    m_conn = conn;
    refresh();
}
bool IPSetting::checkactive()
{
    for (auto const & conn : NetworkManager::activeConnections())
    {
        if((conn->connection() == m_conn) && (conn->state() == NetworkManager::ActiveConnection::Activated))
        {
            return true;
        }
    }
    return  false;
}
void IPSetting::refresh()
{
    clear();

//    auto settings = m_conn.data()->settings();
    if(checkactive()){
        NetworkManager::ConnectionSettings::Ptr connectionSettings = m_conn->settings();
        if(bipv4){
            NetworkManager::Ipv4Setting::Ptr ipv4Setting = connectionSettings->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();

            auto method = ipv4Setting->method();

            if(method == NetworkManager::Ipv4Setting::Disabled)
            {
                ui->combox_method->setCurrentIndex(2);
            }
            else if(method == NetworkManager::Ipv4Setting::Manual)
            {
                ui->combox_method->setCurrentIndex(1);
                auto iplist = ipv4Setting->addresses();
                if(iplist.count() > 0) {
                    ui->ip->setText(iplist.first().ip().toString());
                    ui->_mask->setText(iplist.first().netmask().toString());
                }
                
                ui->_gateway->setText(ipv4Setting->gateway());
            }
            else {
                ui->combox_method->setCurrentIndex(0);
            }

            if(ipv4Setting->dns().size() >= 1)
            {
                ui->dns1->setText(ipv4Setting->dns().at(0).toString());
            }
            if(ipv4Setting->dns().size() >= 2)
                ui->dns2->setText(ipv4Setting->dns().at(1).toString());

        } else {
            NetworkManager::Ipv6Setting::Ptr ipv6Setting = connectionSettings->setting(NetworkManager::Setting::Ipv6).dynamicCast<NetworkManager::Ipv6Setting>();
            auto method = ipv6Setting->method();
            if(method == NetworkManager::Ipv6Setting::Ignored)
            {
                ui->combox_method->setCurrentIndex(2);
            }
            else if(method == NetworkManager::Ipv6Setting::Manual)
            {
                ui->combox_method->setCurrentIndex(1);
                auto iplist = ipv6Setting->addresses();
                if(iplist.count() > 0) {
                    ui->ip->setText(iplist.first().ip().toString());
                    ui->_mask->setText(QString::number(iplist.first().prefixLength()));
                    ui->_gateway->setText(iplist.first().gateway().toString());
                }
                

            }
            else {
                ui->combox_method->setCurrentIndex(0);
            }

            ui->_gateway->setPlaceholderText(tr("MustInput"));
            if(ipv6Setting->dns().size() >= 1)
            {
                ui->dns1->setText(ipv6Setting->dns().at(0).toString());
            }
            if(ipv6Setting->dns().size() >= 2)
                ui->dns2->setText(ipv6Setting->dns().at(1).toString());
        }
    }

}

void IPSetting::onMethodIndexChanged(int index)
{
    switch (index) {
    case 0:
    {
        ui->ip_address->setVisible(false);
        ui->mask->setVisible(false);
        ui->gateway->setVisible(false);
        ui->f_dns->setVisible(true);
        ui->sec_dns->setVisible(true);
        break;
    }
    case 1:
    {
        ui->ip_address->setVisible(true);
        ui->mask->setVisible(true);
        ui->gateway->setVisible(true);
        ui->f_dns->setVisible(true);
        ui->sec_dns->setVisible(true);
        break;
    }
    case 2:
    {
        ui->ip_address->setVisible(false);
        ui->mask->setVisible(false);
        ui->gateway->setVisible(false);
        ui->f_dns->setVisible(false);
        ui->sec_dns->setVisible(false);
        break;
    }
    default:
        break;
    }
}

void IPSetting::clear()
{
    ui->ip->setText("");
    ui->_mask->setText("");
    ui->_gateway->setText("");
    ui->dns1->setText("");
    ui->dns2->setText("");
}
bool IPSetting::save()
{
    NetworkManager::ConnectionSettings::Ptr connectionSettings = m_conn->settings();

    if(bipv4){
        NetworkManager::Ipv4Setting::Ptr ipv4Setting = connectionSettings->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();
        QList<NetworkManager::IpAddress> addrs = ipv4Setting->addresses();
        if(ui->combox_method->currentIndex() == 0){
            ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Automatic);
            ipv4Setting->setGateway("");
            addrs.clear();
            //auto
        } else if(ui->combox_method->currentIndex() == 1){
            //Manual
            ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Manual);

            NetworkManager::IpAddress addr;
            QString ipaddress = ui->ip->text();
            QString mask  = ui->_mask->text();
            QString gateway = ui->_gateway->text();
            addr.setIp(QHostAddress(ipaddress));
            addr.setGateway(QHostAddress(gateway));
            addr.setNetmask(QHostAddress(mask));
            if(addr.ip().toString() != ipaddress)
            {
                QMessageBox::information(this,tr("IPV4 Error"),tr("IP Param Error")+tr("ip address error"));
                return false;
            }
            if(addr.netmask().toString() != mask)
            {
                QMessageBox::information(this,tr("IPV4 Error"),tr("IP Param Error")+tr("mask error"));
                return false;
            }
            if(addr.gateway().toString() != gateway)
            {
                QMessageBox::information(this,tr("IPV4 Error"),tr("IP Param Error")+tr("gateway error"));
                return false;
            }
            if(addrs.isEmpty())
            {
                addrs.push_back(addr);
            }
            else
            {
                addrs.replace(0, addr);
            }

        } else {
            //diable
            ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Disabled);
            ipv4Setting->setGateway("");
            ipv4Setting->setAddressData(NMVariantMapList());
            addrs.clear();
        }
        ipv4Setting->setAddresses(addrs);

        QList<QHostAddress> ipv4_dns;
        if(QHostAddress().setAddress(ui->dns1->text()))
            ipv4_dns.push_back(QHostAddress(ui->dns1->text()));
        if(QHostAddress().setAddress(ui->dns2->text()))
            ipv4_dns.push_back(QHostAddress(ui->dns2->text()));
        ipv4Setting->setDns(ipv4_dns);

    } else {
        NetworkManager::Ipv6Setting::Ptr ipv6Setting = connectionSettings->setting(NetworkManager::Setting::Ipv6).dynamicCast<NetworkManager::Ipv6Setting>();
        QList<NetworkManager::IpAddress> addrs2 = ipv6Setting->addresses();
        if(ui->combox_method->currentIndex()==0){
            ipv6Setting->setMethod(NetworkManager::Ipv6Setting::Automatic);
            addrs2.clear();
        } else if(ui->combox_method->currentIndex() == 1) {
            NetworkManager::IpAddress addr;

            QString ipaddress = ui->ip->text();
            QString mask  = ui->_mask->text();
            QString gateway = ui->_gateway->text();
            addr.setIp(QHostAddress(ipaddress));
            addr.setGateway(QHostAddress(gateway));

            if(addr.ip().toString() != ipaddress)
            {
                QMessageBox::information(this,tr("IPV6 Error"),tr("IP Param Error")+tr("ip address error"));
                return false;
            }
            if(!QHostAddress().setAddress(mask))
            {
                QMessageBox::information(this,tr("IPV6 Error"),tr("IP Param Error")+tr("before error"));
                return false;
            }
            if(addr.gateway().toString() != gateway)
            {
                QMessageBox::information(this,tr("IPV6 Error"),tr("IP Param Error")+tr("gateway error"));
                return false;
            }

            ipv6Setting->setMethod(NetworkManager::Ipv6Setting::Manual);

            if(addrs2.isEmpty())
            {
                addr.setPrefixLength(mask.toInt());
                addrs2.push_back(addr);
            }
            else
            {
                addrs2.replace(0, addr);
            }
//                NetInterface::setipv6dns(intername,dns1,dns2);

        } else {
            ipv6Setting->setMethod(NetworkManager::Ipv6Setting::Ignored);
            ipv6Setting->setAddressData(NMVariantMapList());
            addrs2.clear();
            
        }
        ipv6Setting->setAddresses(addrs2);
        QList<QHostAddress> ipv6_dns;
        if(QHostAddress().setAddress(ui->dns1->text()))
            ipv6_dns.push_back(QHostAddress(ui->dns1->text()));
        if(QHostAddress().setAddress(ui->dns2->text()))
            ipv6_dns.push_back(QHostAddress(ui->dns2->text()));
        ipv6Setting->setDns(ipv6_dns);
    }

    QDBusPendingReply<> reply = m_conn->update(connectionSettings->toMap());
    reply.waitForFinished();
    if(!reply.isValid())
    {
        QMessageBox::information(this,tr("Network Config Error"),tr("Please input right config."));
        return false;
    }
    m_conn->save();
    QString command = QString::fromUtf8("nmcli device reapply \"%1\"").arg(intername);
    QProcess::startDetached(command);
    return true;
}
