#include "netinterface.h"
#include <QtNetwork/QNetworkInterface>
#include <QDebug>
#include <QProcess>
#include <NetworkManagerQt/WirelessNetwork>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include <NetworkManagerQt/WirelessSetting>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/Ipv4Setting>
#include <NetworkManagerQt/Ipv6Setting>
#include <NetworkManagerQt/Utils>
#include <NetworkManagerQt/WiredDevice>
#include <QFile>
#include <QThread>
NetInterface::NetInterface(QObject *parent) : QObject(parent)
{
    init();
}
void NetInterface::init()
{
    netlist.clear();
    auto interlist = QNetworkInterface::allInterfaces();
    for (int i = 0; i < interlist.size(); i++) {
        auto interface = interlist.at(i);
        if(interface.type() == QNetworkInterface::Ethernet){
            NetDetailInfo * info = new NetDetailInfo;
            QString netname = interface.humanReadableName();
            info->setNetName(netname);
            info->setName(netname);
            QString mac = interface.hardwareAddress();

            info->setMac(mac);
            auto address = interface.allAddresses();
            for (int j = 0; j < address.size(); j++) {
                auto item = address.at(j);
                if(item != QHostAddress::LocalHost && item.toIPv4Address()){
                    QString ip =item.toString();
                    info->setIP_V4(ip);
                }
                if(item != QHostAddress::LocalHost && item.protocol()==QAbstractSocket::IPv6Protocol && (!item.isLoopback())){
                    QString ip =item.toString();
                    ip = ip.mid(0,ip.indexOf("%"));
                    info->setIP_V6(ip);
                }
            }
            auto entry = interface.addressEntries();
            for (int j = 0; j < entry.size(); j++) {
                auto item = entry.at(j);
                info->setMask(item.netmask().toString());
                info->setGateWay(item.broadcast().toString());

            }
            netlist.push_back(info);
        }

    }
}
void NetInterface::setipv4Method(QString name,int method)
{
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
   deleteInterArg(filepath,"BOOTPROTO");
   deleteInterArg(filepath,"ONBOOT");
    QString methodstr;
    switch (method) {
    case 0:
        methodstr = "dhcp";
        setInterfaceArg(filepath,"ONBOOT","yes");
        setInterfaceArg(filepath,"BOOTPROTO",methodstr);
        deleteInterArg(filepath,"IPADDR");
        deleteInterArg(filepath,"NETMASK");
        deleteInterArg(filepath,"GATEWAY");
        break;
    case 1:
        methodstr = "static";
        setInterfaceArg(filepath,"ONBOOT","yes");
        setInterfaceArg(filepath,"BOOTPROTO",methodstr);
        break;
    default:
        setInterfaceArg(filepath,"ONBOOT","no");
        break;
    }
    //set "BOOTPROTO";
    clearemptyline(filepath);
}
void NetInterface::cleardns(QString name)
{
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
    deleteInterArg(filepath,"DNS");
    clearemptyline(filepath);
}

void NetInterface::setname(QString name,QString dev_name)
{
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
    deleteInterArg(filepath,"NAME");
    setInterfaceArg(filepath,"NAME",dev_name);
}
void NetInterface::setdns(QString name,QString key,QString dns)
{
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
    //set "dns1";
    setInterfaceArg(filepath,key,dns);
}
void NetInterface::setenable(QString name,QString flag)
{
//    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
//    //set "BOOTPROTO";
//    deleteInterArg(filepath,"IPADDR");
//    setInterfaceArg(filepath,"ONBOOT",ip);
}

void NetInterface::createfile(NetworkManager::Device::Ptr device)
{
    QString name = device.data()->interfaceName();
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
    QString cmd = QString("sudo touch %1").arg(filepath);
    QProcess process;
    process.start(cmd);
    process.waitForFinished(-1);
    if(QFile::exists(filepath)){
        QProcess process1;
        QString cmd1 = QString("sudo sh -c \" echo \' \' > %1 \"").arg(filepath);
        qInfo()<<cmd1;
        process1.start(cmd1);
        process1.waitForFinished(-1);
    }
    NetworkManager::WiredDevice * wiredDevice = qobject_cast<NetworkManager::WiredDevice *>(device.data());
    QString uuid;

    if(wiredDevice)
    {
        //获取可用的有线连接
        for (auto const & m_wiredConnect : wiredDevice->availableConnections())
        {
            uuid = m_wiredConnect->uuid();

        }
    }
    setInterfaceArg(filepath,"TYPE","Ethernet");
    setInterfaceArg(filepath,"PROXY_METHOD","none");
    setInterfaceArg(filepath,"BROWSER_ONLY","no");
    setInterfaceArg(filepath,"DEFROUTE","yes");
    setInterfaceArg(filepath,"IPV4_FAILURE_FATAL","yes");
    setInterfaceArg(filepath,"IPV6_DEFROUTE","yes");
    setInterfaceArg(filepath,"IPV6_FAILURE_FATAL","no");
    setInterfaceArg(filepath,"UUID",uuid);
    setInterfaceArg(filepath,"NAME",name);
    setInterfaceArg(filepath,"IPV6_AUTOCONF","yes");
    setInterfaceArg(filepath,"DEVICE",name);
    setInterfaceArg(filepath,"IPV6_PRIVACY","no");
    setInterfaceArg(filepath,"PREFIX","24");
    setInterfaceArg(filepath,"ONBOOT","yes");
    setInterfaceArg(filepath,"BOOTPROTO","dhcp");
    setInterfaceArg(filepath,"IPV6INIT","yes");
    setInterfaceArg(filepath,"IPV6_AUTOCONF","yes");
}

void NetInterface::setIPV4(QString name,QString ip,QString mask,QString gateway)
{
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
    //set "BOOTPROTO";
    deleteInterArg(filepath,"IPADDR");
    setInterfaceArg(filepath,"IPADDR",ip);
    //set "NETMASK";
    deleteInterArg(filepath,"NETMASK");
    setInterfaceArg(filepath,"NETMASK",mask);
    //set "IPADDR";
    deleteInterArg(filepath,"GATEWAY");
    setInterfaceArg(filepath,"GATEWAY",gateway);
    clearemptyline(filepath);

}
void NetInterface::setipv6Method(QString name,int method)
{
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
    deleteInterArg(filepath,"IPV6INIT");
    deleteInterArg(filepath,"IPV6_AUTOCONF");
    QString methodstr;
    switch (method) {
    case 0:
        methodstr = "dhcp";
        setInterfaceArg(filepath,"IPV6INIT","yes");
        setInterfaceArg(filepath,"IPV6_AUTOCONF","yes");
        deleteInterArg(filepath,"IPV6ADDR");
        deleteInterArg(filepath,"IPV6_DEFAULTGW");
        break;
    case 1:
        methodstr = "none";
        setInterfaceArg(filepath,"IPV6INIT","yes");
        setInterfaceArg(filepath,"IPV6_AUTOCONF","no");
        break;
    default:
        methodstr = "no";
        setInterfaceArg(filepath,"IPV6INIT","no");
        setInterfaceArg(filepath,"IPV6_AUTOCONF","yes");
        break;
    }
    //set "BOOTPROTO";
    clearemptyline(filepath);
}

void NetInterface::setIPV6(QString name,QString ip,QString mask,QString gateway)
{
    QString filepath(QString("/etc/sysconfig/network-scripts/ifcfg-%1").arg(name));
    //set "IPV6ADDR";
    QString ipadd = ip+"/"+mask;
    deleteInterArg(filepath,"IPV6ADDR");
    setInterfaceArg(filepath,"IPV6ADDR",ipadd);
    //set "IPADDR";
    deleteInterArg(filepath,"IPV6_DEFAULTGW");
    setInterfaceArg(filepath,"IPV6_DEFAULTGW",gateway);
    clearemptyline(filepath);
}

void NetInterface::setInterfaceArg(QString path,QString Key,QString value)
{
    QString item = Key+"=" + value;
    QString cmd = QString("sudo sed -i \"$a %1\" %2").arg(item).arg(path);
    qInfo()<<cmd;
    QProcess process;
    process.start(cmd);
    process.waitForFinished(-1);
    qInfo()<<QString::fromUtf8(process.readAllStandardError());
}

void NetInterface::deleteInterArg(QString path,QString key)
{
    QString cmd = QString("sudo sed -i /^%1/d %2").arg(key).arg(path);
    QProcess process;
    process.start(cmd);
    process.waitForFinished(-1);
}

void NetInterface::clearemptyline(QString filepath)
{
     QString cmd = QString("sudo sed -i /^$/d %1").arg(filepath);
     QProcess process;
     process.start(cmd);
     process.waitForFinished(-1);
}
void NetInterface::restartserver(QString name)
{
    QString cmd1 = QString("sudo ifconfig %1 down").arg(name);
    QProcess process1;
    process1.startDetached(cmd1);
    qInfo()<<cmd1;
    process1.waitForFinished(-1);
    QThread::msleep(500);
    QString cmd2 = QString("sudo systemctl restart NetworkManager");
    QProcess process2;
    process2.startDetached(cmd2);
    qInfo()<<cmd2;
    process2.waitForFinished(-1);
    QThread::msleep(500);
    QString cmd3 = QString("sudo ifconfig %1 up").arg(name);
    QProcess process3;
    process3.startDetached(cmd3);
    qInfo()<<cmd3;
    process3.waitForFinished(-1);

}
