#ifndef MONITOROUTPUTWIDGET_H
#define MONITOROUTPUTWIDGET_H

#include <QWidget>
#include <KScreen/Types>
#include "xrandrbrightness.h"
#include "ui_monitoroutputwidget.h"
#include "monitor.h"

class MonitorOutputWidget : public QWidget
{
    Q_OBJECT
public:
    enum MUTIL_SCREEN_SETTINGS {
        UNIFIED_SCREEN,
        EXTENDED_SCREEN,
        ONLY_1_SCREEN,
        ONLY_2_SCREEN,
        OTHER_SCREEN,
    };
    explicit MonitorOutputWidget(KScreen::ConfigPtr config, QWidget *parent = nullptr);
    ~MonitorOutputWidget();
    KScreen::OutputList mOutputs;
    KScreen::ConfigPtr mConfig;
public slots:
    void positionChanged();
private slots:
    void onResolutionChanged(int index);
    void onOrientationChanged(int index);
    void onEnabledChanged(bool enabled);
    void onRateChanged(int index);
    void onDpiScalePrecentChanged(int index);
    void onPrimaryChanged(bool enabled);
    void onMutilScreenIndexChanged(int index);
    void outputChanged(int index);
    void initOutput();
private:
    void updateDisplay(const KScreen::OutputPtr &output);
    void updateRefreshRates();
    void extended();
    void unified();
    void onlyOne();
    void initMutilScreen();
    int getCurrentDpi();
    int brighttovalue(float);
    float valuetobright(int);
    QList<QSize> getCommonSizes();
private:
    Ui::MonitorOutputWidget *ui;
    XRandrBrightness *mBrightness;
    MUTIL_SCREEN_SETTINGS mCurrentMutilMode;
};

#endif // MONITOROUTPUTWIDGET_H
