/*
    Copyright (C) 2014  P.L. Lucas <selairi@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef _MONITORPICTURE_H_
#define _MONITORPICTURE_H_

#include <QGraphicsView>
#include <QGraphicsRectItem>
#include <QGraphicsTextItem>
#include <QSvgRenderer>
#include <QGraphicsSvgItem>
#include <QDialog>
#include "monitor.h"
#include "ui_monitorpicture.h"
#include "monitoroutputwidget.h"

class MonitorPicture;

class MonitorPictureWidget : public QWidget
{
    Q_OBJECT
public:
    friend class MonitorPicture;
    MonitorPictureWidget(KScreen::ConfigPtr config,  QWidget * parent = nullptr, Qt::WindowFlags f = nullptr);
    ~MonitorPictureWidget();
    void setScene();
    void moveMonitorPictureToCenter(MonitorPicture* monitorPicture);
    void updateScene();
    void updateMonitorWidgets();
    KScreen::OutputList mOutputs;
    KScreen::ConfigPtr mConfig;
signals:
    void positionChanged();
private slots:
    void outputChanged();
private:
    void addOutput(int key, KScreen::OutputPtr output);
    Ui::MonitorPictureWidget ui;
    QList<MonitorPicture*> pictures;
    qreal monitorsWidth, monitorsHeight;
    QGraphicsScene *scene;
};

class MonitorPicture : public QGraphicsRectItem
{
public:
    MonitorPicture(int key, MonitorPictureWidget *monitorPictureWidget);
    qreal originX, originY;
//    qreal scale = 0;
    int mKey = 0;
    void updatePos();
    void setMonitorRotation();
    void setPrimary();
private:
    QGraphicsTextItem *textItem;
    MonitorPictureWidget *monitorWidget;
protected:
    void mouseReleaseEvent(QGraphicsSceneMouseEvent * event);
};
class MonitorPictureProxy: public QObject
{
    Q_OBJECT
    public:
        MonitorPictureProxy(QObject *parent, MonitorPicture *monitorPicture);
        MonitorPicture *monitorPicture;

    public slots:
        void updatePosition();
        void updateRotation();
        void upatePrimary();
};
#endif // _MONITORPICTURE_H_
