#include "categorizeditemdelegate.h"

#include <QPainter>
#include <QDebug>
#include <XdgIcon>
#include <QApplication>
#include "categorizedmodel.h"
CategorizedItemDelegate::CategorizedItemDelegate(QObject *parent) : QStyledItemDelegate(parent)
{

}
CategorizedItemDelegate::~CategorizedItemDelegate()
{

}
void CategorizedItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QString title = index.data(Qt::DisplayRole).toString();

    painter->setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);//消锯齿;
    QStyleOptionViewItem opt = option;
    initStyleOption(&opt, index);
    QRect rect = option.rect;

    QSize iconSize(rect.height()*0.6,rect.height()*0.6);

    qreal dpr = qApp->devicePixelRatio();
    if (dpr < 1) dpr = 1;
    QPixmap pixmap = opt.icon.pixmap(iconSize / dpr);
    if (dpr > 1 && pixmap.size() == iconSize)
        pixmap = opt.icon.pixmap(iconSize);


    if(pixmap.isNull() && title.isNull())
    {
        QRect r = option.rect;
        r.setHeight(18);
        painter->fillRect(r, QColor("#f0f1f2"));
        return;
    }

    painter->fillRect(rect, Qt::white);
    if(option.state & QStyle::State_Selected)
    {
        painter->fillRect(rect, QColor("#2c9dfc"));
        painter->setPen(QColor("#ffffff"));
    } else {
        painter->setPen(QColor("#000000"));
    }
    // 图片显示
    {
        QRect dst = rect;
        dst.setRight(rect.left() +44);
        QRect area(0,0,rect.height()*0.6,rect.height()*0.6);
        area.moveCenter(dst.center());
        painter->drawPixmap(area, pixmap);
    }

    // 文件名显示
    {
        QRect dst = rect;
        dst.setLeft(rect.left() + rect.height()*0.6 +20);
        painter->drawText(dst, Qt::AlignLeft | Qt::AlignVCenter, title);
        if(!(option.state & QStyle::State_Selected)){
            if(!pixmap.isNull() && !title.isEmpty()){
                if(index.row() != index.model()->rowCount(index.parent())-1 ){
                    painter->setPen(QPen(QColor("#e5e5e7"),1));
                    painter->drawLine(dst.bottomLeft(), dst.bottomRight());
                }

            }
        }

    }
}
QSize CategorizedItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QSize size = QStyledItemDelegate::sizeHint(option, index);
     QString title = index.data(Qt::DisplayRole).toString();
//    if(title.compare("用户名")==0){
//        size.setHeight(54);
//        return size;
//    }
    int height = index.data(RowHeight).toInt();
    if(title.isEmpty()){
        if(index.row() == 0) {
            size.setHeight(1);
        } else {
            size.setHeight(18);
        }
    }
    else
        size.setHeight(height);
    return size;
}
