#include <QDBusConnection>
#include <QApplication>
#include <QSettings>
#include <QDBusMessage>
#include "mainwindow.h"
#include "QCommandLineParser"

#include <libfm-qt/libfmqt.h>
#include <QLibraryInfo>
#include <QLocale>
#include <XdgIcon>
#include <KDBusService>

int main(int argc, char **argv)
{
    QApplication app(argc, argv);

    app.setWindowIcon(XdgIcon::fromTheme("settings"));
    app.setOrganizationName(QStringLiteral("nde"));
    app.setApplicationName(QStringLiteral("nde-systemsettings"));
    app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);

    QCommandLineParser parser;
    parser.setApplicationDescription(QStringLiteral("Nde SystemSettings"));
    const QString VERINFO = QStringLiteral(NDE_CONFIG_VERSION
                                           "\nlibnde   " NDE_VERSION
                                           "\nQt        " QT_VERSION_STR);
    app.setApplicationVersion(VERINFO);
    QCommandLineOption moduleOption(QStringLiteral("module"), QObject::tr("Display this Module at first.<eg:systeminfo,screen,proxy,language,datetime,defaultapp...>\n"),
                                    "module", QStringLiteral("systeminfo"));
    parser.addOption(moduleOption);
    parser.addVersionOption();
    parser.addHelpOption();
    parser.process(app);

    //获取默认优先显示的模块
    QString module = QStringLiteral("systeminfo");
    if(parser.isSet(moduleOption))
    {
        module = parser.value(moduleOption);
    }
    //initialize libfm-qt
    Fm::LibFmQt libfmQt;
    app.installTranslator(libfmQt.translator());


    MainWindow w(module);

    KDBusService *service = new KDBusService(KDBusService::Unique, &app);

    //handle external opening
    QObject::connect(service, &KDBusService::activateRequested, &w, [&w](const QStringList &arguments, const QString &/*workingDirectory*/){
        w.wshow();
        if (arguments.isEmpty())
            return ;

        QString module;
        for(int i=0;i<arguments.count();i++)
            module=arguments.at(i);

        if(!module.isEmpty())
        {
            w.setCurrentItem(module);
        }
    });


    QSize s = QSettings{}.value("size").toSize();
    if (!s.isEmpty())
        w.resize(s);
    QPoint pos = QSettings{}.value("pos").toPoint();
    if(!pos.isNull())
        w.move(pos);
    w.show();

    int ret = app.exec();

    QSettings{}.setValue("size", w.size());
    QSettings{}.setValue("pos", w.pos());

    return ret;
}
