#include <QFile>
#include <QStorageInfo>
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>
#include "renamehost.h"
#include "ui_renamehost.h"
#include "../baseui/messagewidget.h"
#include "systeminfoinstance.h"

RenameHost::RenameHost(QWidget *parent) :
    Page(parent),
    ui(new Ui::RenameHost),
    messageW_(nullptr)
{
    ui->setupUi(this);
    QFile f(":/qss/renamehost.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();
}

RenameHost::~RenameHost()
{
    delete ui;
    delete messageW_;
}

void RenameHost::applyBtnClicked()
{
    messageW_->close ();

    QString oldHostName = QSysInfo::machineHostName();

    QString newHostName = ui->hostNameLineEdit->text();
    newHostName = newHostName.simplified();   // 去掉前后空格
    if(newHostName.isEmpty()) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("The host name cannot be empty."));
        return;
    }

    if(newHostName.startsWith("-")) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name cannot start with '-' characher."));
        return;
    }

    if(newHostName.startsWith(".")) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name cannot start with '.' characher."));
        return;
    }

    if(newHostName == "." || newHostName == "..") {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name '%1' is not allowed.").arg(newHostName));
        return;
    }

    if(messageW_->isDigitStr(newHostName)) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Fully numeric name is not allowed."));
        return;
    }
    //判断首位是否为数值
    if(messageW_->isDigitStr(newHostName.left(1))) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name cannot start with numeric."));
        return;
    }

    if(newHostName == "$") {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name '$' is not allowed."));
        return;
    }

    if(newHostName.contains(" ", Qt::CaseSensitive)) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name cannot contain spaces."));
        return;
    }

    //hostname 不能包含'_',只能包含'0-9','a-z','A-Z','.','-'
    //包含'_'时会导致网络(NetworkManager版本NetworkManager-1.20.0-5.el8_1)无法连接，在升级NetworkManager到NetworkManager-1.26.0-12.el8_3.x86_64之后正常
    QRegExp rx("[^a-zA-Z0-9.-]");
    int pos = 0;
    QString matchStr = "";

    while((pos = rx.indexIn(newHostName, pos)) != -1) {
        matchStr += newHostName[pos] + ",";
        pos += rx.matchedLength();
    }

    if(!matchStr.isEmpty()) {
        matchStr = matchStr.left(matchStr.length() - 1);
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name cannot contain character: '%1'.").arg(matchStr));
        return;
    }

    QRegExp rxend(PORTABLE_END_CHARS);
    if(rxend.exactMatch(newHostName.right(1))) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("End of Host name cannot contain special character: '%1'.").arg(newHostName.right(1)));
        return;
    }

    if(newHostName.size() > MAXINPUTSIZE) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 10,
                              tr("Host name must be shorter than 16 characters."));
        return;
    }

    QStringList listDir;
    listDir << "root" << "home" << "daemon" << "system" << "etc" << "boot" << "lib" << "media" << "opt" << "sbin" << "sys"
            << "tmp" << "var" << "bin"<< "dev" << "lib64" << "mnt" << "proc" << "run" << "srv" << "usr";
    if(listDir.contains(newHostName)) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 12,
                              tr("Host name is reserved for system: %1.").arg(newHostName));
        return;
    }

    if(newHostName == oldHostName) {
        messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 12,
                              tr("The host name already exists."));
        return;
    }

    KAuth::Action action("org.nde.auth.helper.changehostname");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));
    action.addArgument(QStringLiteral("hostname"), newHostName);
    KAuth::ExecuteJob *job = action.execute();
    if(job->exec())
    {
        if(newHostName == QSysInfo::machineHostName())
        {
            ui->hostNameLineEdit->setText("");
            SystemInfoInstance::ins()->setHostName(newHostName);
        }
        else
        {
            messageW_->showWidget(ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->hostNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->hostNameLineEdit->height() - 12,
                                  tr("Rename host failed."));
            return;
        }
    }
}

void RenameHost::backBtnClicked()
{
    ui->hostNameLineEdit->setText("");
    messageW_->close();
}

void RenameHost::refreshPage()
{
    //
}

void RenameHost::mousePressEvent(QMouseEvent *)
{
    messageW_->close ();
}

void RenameHost::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    messageW_->close ();
}
