#include "virtualmemory.h"
#include "ui_virtualmemory.h"
#include "../baseui/messagewidget.h"

VirtualMemory::VirtualMemory(QWidget *parent) :
    Page(parent),
    ui(new Ui::VirtualMemory)
{
    ui->setupUi(this);

    //设置样式
    QFile f(":/qss/virtualmemory.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    //虚拟内存配置的开关
    QHBoxLayout *layout = new QHBoxLayout();
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);
    ui->virtual_memory_switch->setLayout(layout);
    _systemVirtualMemorySwitchToggle = new NdeWidget::QMaterialToggle(ui->virtual_memory_switch);
    bool virtualMemorySwitchStatus = QFile::exists(VIRTUAL_MEMORY_FILE_NAME);
    _systemVirtualMemorySwitchToggle->setChecked(virtualMemorySwitchStatus);
    ui->virtual_memory_size_frame->setVisible(virtualMemorySwitchStatus);
    connect(_systemVirtualMemorySwitchToggle, &NdeWidget::QMaterialToggle::toggled, this, &VirtualMemory::onSystemVirtualMemorySwitchToggle);
    layout->addWidget(_systemVirtualMemorySwitchToggle);

    //限制-90，90，并限定小数点后4位--限制输入框
    QRegExp rx("^(90|[1-8]?\\d(\\.\\d{1,2})?)$");
    QRegExpValidator *pReg = new QRegExpValidator(rx, this);
    ui->virual_memory_size_line->setValidator(pReg);

    //初始化信息弹窗
    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();

    connect(_systemVirtualMemorySwitchToggle, &NdeWidget::QMaterialToggle::toggled, this, [this](){
           messageW_->close();
    });
    connect(ui->virual_memory_size_line, &QLineEdit::textChanged, this, [this](){
           messageW_->close();
    });
}

VirtualMemory::~VirtualMemory()
{
    delete ui;
    delete messageW_;
}

void VirtualMemory::applyBtnClicked()
{
    if(mVirtualMemorySWitchState){

        //当前页面修改内存大小，如果已经存在之前的设置，则清除旧配置，新配置即刻生效
        QFileInfo file(VIRTUAL_MEMORY_FILE_NAME);
        if(file.exists()){
            disableVirtualMemory();
        }
        enableVirtualMemory();
    }else{
        disableVirtualMemory();
    }
    informationTips();
}

void VirtualMemory::backBtnClicked()
{
    messageW_->close();
}

void VirtualMemory::refreshPage()
{
    QFileInfo file(VIRTUAL_MEMORY_FILE_NAME);
    if(file.exists()){
        QString swapSize = QString::number(file.size()/(1024 * 1024 * 1024 * 1.0),'f',2);
        ui->virual_memory_size_line->setText(swapSize);
    }else{
        ui->virual_memory_size_line->setText(0);
    }
}

void VirtualMemory::onSystemVirtualMemorySwitchToggle(bool checked)
{
    ui->virtual_memory_size_frame->setVisible(checked);
    mVirtualMemorySWitchState = checked;
}

bool VirtualMemory::enableVirtualMemory()
{

    float virtualMemorySize = 0;
    QString virtualMemoryCommand;
    bool resultCommand;

    QProcess *virtualMemory = new QProcess(this);
    bool systemVirtualMemoryResult = false;
    QStorageInfo storage = QStorageInfo::root();
    //系统空闲容量
    qint64 rootFree = storage.bytesFree() > storage.bytesAvailable() ? storage.bytesAvailable() : storage.bytesFree();
    //单位为G
    float systemFree = rootFree/(1024 * 1024 * 1024 * 1.0);

    virtualMemorySize = ui->virual_memory_size_line->text().toFloat();

    qDebug()<<"Start virtual memory!";
    //检测磁盘可用空间大小
    if(virtualMemorySize > 0 && virtualMemorySize < systemFree){
        //使用fallocate替换dd
         virtualMemoryCommand = QString::fromUtf8("sudo fallocate -l %1G %2").arg(virtualMemorySize).arg(VIRTUAL_MEMORY_FILE_NAME);
         virtualMemory->start(virtualMemoryCommand);
         resultCommand = virtualMemory->waitForFinished(-1);

        //检测文件是否创建成功
        if(resultCommand){
            virtualMemoryCommand = QString::fromUtf8("sudo mkswap %1").arg(VIRTUAL_MEMORY_FILE_NAME);
            virtualMemory->start(virtualMemoryCommand);
            resultCommand = virtualMemory->waitForFinished(-1);
            if(resultCommand){
                virtualMemoryCommand = QString::fromUtf8("sudo swapon %1").arg(VIRTUAL_MEMORY_FILE_NAME);
                virtualMemory->start(virtualMemoryCommand);
                resultCommand = virtualMemory->waitForFinished();
                if(resultCommand){
                    qDebug()<<"the virtual memory is successfully started!";
                    //配置开机自启动---通过文件读写的方式进行追加
                     QFile *file =new QFile(VIRTUAL_MEMORY_AUTOMATIC_MOUNT_CONFIGURATION);
                     virtualMemoryCommand = QString::fromUtf8("sudo chmod 777 %1").arg(VIRTUAL_MEMORY_AUTOMATIC_MOUNT_CONFIGURATION);
                     virtualMemory->start(virtualMemoryCommand);
                     resultCommand = virtualMemory->waitForFinished();
                     bool ok = file->open(QIODevice::ReadWrite|QIODevice::Append);
                     if(ok){
                         QTextStream txtOutput(file);
                         virtualMemoryCommand = QString::fromUtf8("%1 swap swap default 0 0").arg(VIRTUAL_MEMORY_FILE_NAME);
                         txtOutput << virtualMemoryCommand << endl;
                         systemVirtualMemoryResult = true;
                         mVirtualMemoryResult = STARTSUCCESSFUL;
                     }
                     file->close();
                     virtualMemoryCommand = QString::fromUtf8("sudo chmod 644 %1").arg(VIRTUAL_MEMORY_AUTOMATIC_MOUNT_CONFIGURATION);
                     resultCommand = QProcess::startDetached(virtualMemoryCommand);
                }
            }else{
                mVirtualMemoryResult = SWAPONFAIL;
                qDebug()<<"Failed to start the virtual memory!";

            }

        }else{
            mVirtualMemoryResult = FALLOCATEFAIL;
            qDebug()<<"Failed to create virtual memory file!";
        }
    }else{
        //如果超过磁盘空闲大小，则不允许创建,并提示用户
        if(virtualMemorySize <= 0){
            mVirtualMemoryResult = INVALIDVALUES;
            qDebug()<<"The virtual memory size Settings are invalid!";
        }else{
            mVirtualMemoryResult = INSUFFICIENTDISKSPACE;
        }
    }
    delete  virtualMemory;
    return systemVirtualMemoryResult;
}

bool VirtualMemory::disableVirtualMemory()
{
    QString virtualMemoryCommand;
    bool resultCommand;

    QProcess *virtualMemory = new QProcess(this);

    qDebug()<<"Disabling virtual memory!";
    virtualMemoryCommand = QString::fromUtf8("sudo swapoff %1").arg(VIRTUAL_MEMORY_FILE_NAME);
    resultCommand = QProcess::startDetached(virtualMemoryCommand);
    if(resultCommand){
        //sed修改/etc/fstab文件，将之前写入的删除 sudo sed -i '/\/var\/swapfile/'d /etc/fstab
        virtualMemoryCommand = QString::fromUtf8("sudo sed -i /\\/var\\/swapfile/\d %1").arg(VIRTUAL_MEMORY_AUTOMATIC_MOUNT_CONFIGURATION);
        resultCommand = QProcess::startDetached(virtualMemoryCommand);
    }else{
        qDebug()<<"Failed to shut down virtual memory!";
    }

     //删除swap文件
    qDebug()<<"delected /var/swapfile failded!";
    int tryCount = 0;
    while(QFile::exists(VIRTUAL_MEMORY_FILE_NAME)){
        if(tryCount >= 5)
            break;
        virtualMemoryCommand = QString::fromUtf8("sudo rm -f %1").arg(VIRTUAL_MEMORY_FILE_NAME);
        virtualMemory->start(virtualMemoryCommand);
        resultCommand = virtualMemory->waitForFinished();
        tryCount++;
    }

    mVirtualMemoryResult = CLOSESUCCESSFUL;

    delete  virtualMemory;
    return true;
}

void VirtualMemory::informationTips()
{
    //控制提示框的方向和提示语句
    QString informationText;
    bool direction;

    switch (mVirtualMemoryResult){
    case STARTSUCCESSFUL:
        //启动成功
        direction = true;
        informationText = tr("The virtual memory is set successfully.");
        break;
    case CLOSESUCCESSFUL:
        //关闭成功
        ui->virtual_memory_size_frame->setVisible(false);
        ui->virual_memory_size_line->clear();
        direction = false;
        informationText = tr("The virtual memory is shut down successfully.");
        break;
    case SWAPONFAIL:
        //启动分区失败
        direction = false;
        ui->virual_memory_size_line->clear();
        _systemVirtualMemorySwitchToggle->setChecked(false);
        ui->virtual_memory_size_frame->setVisible(false);
        informationText = tr("Failed to start the virtual memory.");
        break;
    case FALLOCATEFAIL:
        //创建分区文件失败
        direction = false;
        ui->virual_memory_size_line->clear();
        _systemVirtualMemorySwitchToggle->setChecked(false);
        ui->virtual_memory_size_frame->setVisible(false);
        informationText = tr("Failed to create virtual memory file.");
        break;
    case INSUFFICIENTDISKSPACE:
        //磁盘空间不足
        direction = false;
        ui->virual_memory_size_line->clear();
        _systemVirtualMemorySwitchToggle->setChecked(false);
        ui->virtual_memory_size_frame->setVisible(false);
        informationText = tr("The system disk space is insufficient.");
        break;
    case INVALIDVALUES:
        //设置值无效
        direction = false;
        ui->virual_memory_size_line->clear();
        _systemVirtualMemorySwitchToggle->setChecked(false);
        ui->virtual_memory_size_frame->setVisible(false);
        informationText = tr("The virtual memory size Settings are invalid.");
        break;
    default:
        break;
    }

    message(direction,informationText);
}

void VirtualMemory::message(bool direction, QString message)
{

    if(direction){
        messageW_->setDirection(MessageWidget::Direction::up);
        messageW_->showWidget(ui->virual_memory_size_line->mapTo(this, QPoint(0, 0)).x()+20,
                              ui->virual_memory_size_line->mapTo(this, QPoint(0, 0)).y() + ui->virual_memory_size_line->height() - 10,
                              message);
    }else{
        messageW_->setDirection(MessageWidget::Direction::right);
        messageW_->showWidget(ui->virtual_memory_switch->mapTo(this, QPoint(0, 0)).x()+20,
                              ui->virtual_memory_switch->mapTo(this, QPoint(0, 0)).y() + ui->virtual_memory_switch->height() - 10,
                              message);
    }
}
