#include "newwifiwidget.h"
#include "ui_newwifiwidget.h"
#include <QListView>
#include "nmconfig.h"
#include <QtConcurrent/QtConcurrent>
#include "../baseui/common.h"

NewWifiWidget::NewWifiWidget(QWidget *parent) :
    Page(parent),
    ui(new Ui::NewWifiWidget)
{
    ui->setupUi(this);
    QFile f(":/qss/newwifiwidget.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    connect(ui->comboBox_type, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &NewWifiWidget::onComboIndexChanged);
    connect(ui->edit_password, &QLineEdit::textChanged, this, &NewWifiWidget::onPasswordTextChanged);
    connect(ui->edit_password, &QLineEdit::textEdited, this, &NewWifiWidget::onPasswordTextChanged);
    connect(ui->edit_name, &QLineEdit::textChanged, this, &NewWifiWidget::onPasswordTextChanged);
    ui->comboBox_type->setView(new QListView());
}

NewWifiWidget::~NewWifiWidget()
{
    delete ui;
}

void NewWifiWidget::initUi(QString uni)
{
    m_device = uni;
    ui->comboBox_type->setCurrentIndex(0);
    ui->edit_name->clear();
    ui->edit_password->clear();
    ui->widget_password->setVisible(false);
    ui->label_msg->setVisible(false);
}

void NewWifiWidget::applyBtnClicked()
{
    ui->label_msg->setVisible(false);
    bool bSuccess = true;
    if(ui->comboBox_type->currentIndex() != 0)
    {
        if(ui->edit_password->text().isEmpty())
        {
            bSuccess = false;
            ui->edit_password->setStyleSheet("border:1 solid red;background-color: rgb(255, 255, 255);");
        }
    }
    if(ui->edit_name->text().isEmpty())
    {
        bSuccess = false;
        ui->edit_name->setStyleSheet("border:1 solid red;background-color: rgb(255, 255, 255);");
    }
    if(!bSuccess)
        return;
    QString name = ui->edit_name->text();
    QString pass = ui->edit_password->text();
    QString type;
    switch (ui->comboBox_type->currentIndex()) {
    case 0:
        type = "sae";
        break;
    case 1:
        type = "none";
        break;
    case 2:
        type = "wpa-psk";
        break;
    case 3:
        type = "wpa-eap";
        break;
    case 4:
        type = "ieee8021x";
        break;
    }
    Common::ChattrDir chat;
    QFuture<int> fun = QtConcurrent::run([name, type, pass, this]()->int{
        QString command = QString::fromUtf8("nmcli connect delete \"%1\"").arg(name);
        QProcess::execute(command);
        command = QString("nmcli connection add con-name \"%1\" ifname \"%2\" "
                                  "autoconnect yes type wifi ssid \"%3\" wifi-sec.key-mgmt \"%4\"")
                            .arg(name).arg(m_device).arg(name).arg(type);
        if(type != "sae")
        {
            command += QString(" wifi-sec.psk \"%1\"").arg(pass);
            //保存一下密钥
            NMConfig::global()->saveWIFIPasswd(name, pass);
        }
        QProcess::execute(command);

        command = QString("nmcli c up \"%1\"").arg(name);
        ui->label_msg->setText(tr("Connecting..."));
        ui->label_msg->setVisible(true);
        return QProcess::execute(command);
    });
    while (fun.isRunning()) {
        QThread::msleep(100);
        QApplication::processEvents();
    }
    if(fun.result() == 0)
    {
        ui->label_msg->setVisible(false);
        Q_EMIT backPage();
    }
    else {
        ui->label_msg->setText(tr("Connect Fail!"));
        ui->label_msg->setVisible(true);
        QString command = QString::fromUtf8("nmcli connect delete \"%1\"").arg(name);
        QProcess::execute(command);
    }
}

void NewWifiWidget::backBtnClicked()
{

}

void NewWifiWidget::refreshPage()
{
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
}

void NewWifiWidget::onComboIndexChanged(int index)
{
    switch (index) {
    case 0:
        ui->widget_password->setVisible(false);
        Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
        break;
    case 1:
    case 2:
    case 3:
    case 4:
        ui->widget_password->setVisible(true);
        ui->edit_password->clear();
        Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
        break;
    default:
        ui->widget_password->setVisible(false);
        break;
    }
}

void NewWifiWidget::onPasswordTextChanged(const QString& text)
{
    if(text.length() > 0)
    {
        QLineEdit* edit = static_cast<QLineEdit*>(sender());
        edit->setStyleSheet("border:0 solid red;background-color: rgb(255, 255, 255);");
    }
    if(ui->widget_password->isVisible())
    {
        if(ui->edit_password->text().size() >= 8)
            Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
        else {
            Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
        }
    }
    else {
        Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
    }
}
