#include "nmconfig.h"
#include <QTimer>
#include <QDebug>
#include <QProcess>
#include <QFileInfo>

#define KEY_HEX "ea462947a76af8a50dba0ad01455c8c3"

NMConfig::NMConfig(QObject *parent) : QObject(parent)
{
    m_bf = new QBlowfish(KEY_HEX);
    m_bf->setPaddingEnabled(true); // Enable padding to be able to encrypt an arbitrary length of bytes

    m_settings = new Nde::Settings(QLatin1String("nde-networkmanager"), this);
    connect(m_settings, &Nde::Settings::settingsChanged, this, &NMConfig::onSettingsChanged);
    qWarning() << m_settings->fileName();

    QTimer::singleShot(0, [this](){
        onSettingsChanged();
    });

    m_interface = new QDBusInterface(QStringLiteral("org.freedesktop.Notifications"),
                                     QStringLiteral("/org/freedesktop/Notifications"),
                                     QStringLiteral("org.freedesktop.Notifications"),
                                     QDBusConnection::sessionBus(),
                                     this);
}

NMConfig *NMConfig::global()
{
    static NMConfig config ;
    return &config;
}

void NMConfig::removeWiFIPasswd(const QString connName)
{
    m_settings->beginGroup("wifi");
    m_settings->remove(connName);
    m_settings->endGroup();
}

QString NMConfig::readWIFIPasswd(const QString connName)
{
    m_settings->beginGroup("wifi");
    QByteArray pw = m_settings->value(connName, "").toByteArray();
    m_settings->endGroup();

    QByteArray decryptedBa = m_bf->decrypted(pw);
    QString decryptedString = QString::fromUtf8(decryptedBa.constData(), decryptedBa.size());
    qDebug() << "Decrypted string:" << decryptedString;

    return decryptedString;
}

void NMConfig::saveWIFIPasswd(const QString connName, const QString passwd)
{
    QByteArray cipherPw = m_bf->encrypted(passwd.toUtf8());
    qDebug() << "Encrypted ba (hex):" << cipherPw;

    m_settings->beginGroup("wifi");
    m_settings->setValue(connName, cipherPw);
    m_settings->endGroup();
}

void NMConfig::notifyMsg(QList<QVariant> args)
{
    m_interface->callWithArgumentList(QDBus::AutoDetect, "Notify", args);
}

void NMConfig::onSettingsChanged()
{
    //
}
