#include "wifiinfowidget.h"
#include "ui_wifiinfowidget.h"
#include <NetworkManagerQt/AccessPoint>
#include <NetworkManagerQt/Ipv4Setting>
#include <NetworkManagerQt/Ipv6Setting>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/WirelessSetting>
#include "nmconfig.h"
#include <arpa/inet.h>
#include <QScrollBar>
#include <QListView>
#include "../baseui/common.h"
#include "../baseui/baseuitr.h"
#include <QDBusMetaType>
#include <QUuid>

#define PAGE_BUTTOM_BTN_FORGET (Page::PAGE_BBTN_USER+1)

#define LMOVE(m,n) ((m)<<(n))
#define RMOVE(m,n) ((m)>>(n))

WifiInfoWidget::WifiInfoWidget(QWidget *parent) :
    Page(parent),
    ui(new Ui::WifiInfoWidget)
{
    ui->setupUi(this);
    QFile f(":/qss/wifiinfowidget.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    connect(ui->comboBox_IPV4, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &WifiInfoWidget::onIPV4MethodIndexChanged);
    connect(ui->comboBox_IPV6, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &WifiInfoWidget::onIPV6MethodIndexChanged);
    connect(ui->edit_IPV6Ip, &QLineEdit::textChanged, this, &WifiInfoWidget::onIPV6Textchanged);
    connect(ui->edit_IPV6Mask, &QLineEdit::textChanged, this, &WifiInfoWidget::onIPV6Textchanged);
    connect(ui->edit_IPV6Getway, &QLineEdit::textChanged, this, &WifiInfoWidget::onIPV6Textchanged);
    connect(ui->edit_IPV6DNS1, &QLineEdit::textChanged, this, &WifiInfoWidget::onIPV6Textchanged);
    connect(ui->edit_IPV6DNS2, &QLineEdit::textChanged, this, &WifiInfoWidget::onIPV6Textchanged);
//    ui->horizontalWidget_11->setVisible(false);
    ui->comboBox_IPV4->setView(new QListView());
    ui->comboBox_IPV6->setView(new QListView());
}

WifiInfoWidget::~WifiInfoWidget()
{
    delete ui;
}

void WifiInfoWidget::initInfo(QString uni, QString ssid)
{
    clear();

    m_device = uni;
    m_ssid = ssid;
    NetworkManager::WirelessDevice* device = nullptr;
    for (auto const & tmp : NetworkManager::networkInterfaces())
    {
        //无线设备
        if(tmp->type() == NetworkManager::Device::Wifi && tmp->interfaceName() == uni)
        {
            device = qobject_cast<NetworkManager::WirelessDevice *>(tmp.data());
            break;
        }
    }
    if(device == nullptr) return;
    NetworkManager::WirelessNetwork::Ptr network = device->findNetwork(ssid);
    if(network.isNull()) return;
    NetworkManager::Connection::Ptr conn;
    for (auto const & dev_conn : device->availableConnections())
    {
        if (dev_conn->settings()->id() == ssid)
        {
            conn = dev_conn;
        }
    }
    if(conn && conn->isValid())
    {
        ui->widget_info->setVisible(true);

        NetworkManager::ConnectionSettings::Ptr connectionSettings = conn->settings();
        NetworkManager::WirelessSetting::Ptr wirelessSetting = connectionSettings->setting(NetworkManager::Setting::Wireless).dynamicCast<NetworkManager::WirelessSetting>();
        NetworkManager::WirelessSecuritySetting::Ptr wifiSecurity = connectionSettings->setting(NetworkManager::Setting::WirelessSecurity).dynamicCast<NetworkManager::WirelessSecuritySetting>();
        NetworkManager::Ipv4Setting::Ptr ipv4Setting = connectionSettings->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();
        NetworkManager::Ipv6Setting::Ptr ipv6Setting = connectionSettings->setting(NetworkManager::Setting::Ipv6).dynamicCast<NetworkManager::Ipv6Setting>();

        ui->label_ssid->setText(m_ssid);
        if(!ipv4Setting->addresses().isEmpty())
        {
            ui->edit_IPV4Ip->setIP(ipv4Setting->addresses().first().ip().toString());
            ui->edit_IPV4Mask->setIP(ipv4Setting->addresses().first().netmask().toString());
            ui->label_IPV4Addr2->setText(ipv4Setting->addresses().first().ip().toString());
        }
        ui->edit_IPV4Getway->setIP(ipv4Setting->gateway());
        if(ipv4Setting->dns().size() >= 1)
        {
            ui->label_DNS->setText(ipv4Setting->dns().at(0).toString());
            ui->edit_IPV4DNS1->setIP(ipv4Setting->dns().at(0).toString());
        }
        if(ipv4Setting->dns().size() >= 2)
            ui->edit_IPV4DNS2->setIP(ipv4Setting->dns().at(1).toString());


        if(!ipv6Setting->addresses().isEmpty())
        {
            ui->edit_IPV6Ip->setText(ipv6Setting->addresses().first().ip().toString());
            ui->edit_IPV6Mask->setText(QString::number(ipv6Setting->addresses().first().prefixLength()));
            if(ipv6Setting->addresses().first().gateway().toString() != "::")
                ui->edit_IPV6Getway->setText(ipv6Setting->addresses().first().gateway().toString());
            ui->label_IPV6Addr->setText(ipv6Setting->addresses().first().ip().toString());
        }
        if(ipv6Setting->dns().size() >= 1)
        {
            if(ui->label_DNS->text().isEmpty())
                ui->label_DNS->setText(ipv6Setting->dns().at(0).toString());
            ui->edit_IPV6DNS1->setText(ipv6Setting->dns().at(0).toString());
        }
        if(ipv6Setting->dns().size() >= 2)
            ui->edit_IPV6DNS2->setText(ipv6Setting->dns().at(1).toString());

        ui->label_protocol->setText(wirelessSetting->security());
        switch (wirelessSetting->band()) {
        case NetworkManager::WirelessSetting::Automatic:
            ui->label_netRate->setText(tr("Auto"));
        break;
        case NetworkManager::WirelessSetting::A:
            ui->label_netRate->setText(tr("5GHz"));
        break;
        case NetworkManager::WirelessSetting::Bg:
            ui->label_netRate->setText(tr("2.4GHz"));
        break;
        }
        switch(wifiSecurity->keyMgmt())
        {
        case NetworkManager::WirelessSecuritySetting::Wep:
            ui->label_securityType->setText("Wep");
            break;
        case NetworkManager::WirelessSecuritySetting::Ieee8021x:
            ui->label_securityType->setText("Ieee8021x");
            break;
        case NetworkManager::WirelessSecuritySetting::WpaNone:
            ui->label_securityType->setText("WPA-NONE");
            break;
        case NetworkManager::WirelessSecuritySetting::WpaPsk:
            ui->label_securityType->setText("WPA-PSK");
            break;
        case NetworkManager::WirelessSecuritySetting::WpaEap:
            ui->label_securityType->setText("WPA-EAP");
            break;
        default:
            ui->label_securityType->setText("Unknown");
            break;
        }
        ui->label_netChannel->setText(QString::number(NetworkManager::findChannel(network->referenceAccessPoint()->frequency())));
        ui->label_mac->setText(network->referenceAccessPoint()->hardwareAddress());
        ui->label_speed->setText(QString("%1 Mbit").arg(network->referenceAccessPoint()->maxBitRate() / 1000));

        if(ipv4Setting->method() == NetworkManager::Ipv4Setting::Automatic)
        {
            NetworkManager::Dhcp4Config::Ptr dhcp4 = device->dhcp4Config();
            if(!dhcp4.isNull())
            {
                ui->label_IPV4Addr2->setText(dhcp4->optionValue("ip_address"));
            }
            ui->comboBox_IPV4->setCurrentIndex(0);
            onIPV4MethodIndexChanged(0);
        }
        else if(ipv4Setting->method() == NetworkManager::Ipv4Setting::Manual){
            ui->comboBox_IPV4->setCurrentIndex(1);
            onIPV4MethodIndexChanged(1);
        }
        else if(ipv4Setting->method() == NetworkManager::Ipv4Setting::Disabled){
            ui->comboBox_IPV4->setCurrentIndex(2);
            onIPV4MethodIndexChanged(2);
        }

        if(ipv6Setting->method() == NetworkManager::Ipv6Setting::Automatic)
        {
            NetworkManager::IpConfig ip6Config = device->ipV6Config();
            if (ip6Config.isValid()) {
                NetworkManager::IpAddress address = ip6Config.addresses().at(0);
                ui->label_IPV6Addr->setText(address.ip().toString());
            }
            ui->comboBox_IPV6->setCurrentIndex(0);
            onIPV6MethodIndexChanged(0);
        }
        else if(ipv6Setting->method() == NetworkManager::Ipv6Setting::Manual){
            ui->comboBox_IPV6->setCurrentIndex(1);
            onIPV6MethodIndexChanged(1);
        }
        else if(ipv6Setting->method() == NetworkManager::Ipv6Setting::Ignored){
            ui->comboBox_IPV6->setCurrentIndex(2);
            onIPV6MethodIndexChanged(2);
        }
    }
    else {
        ui->widget_info->setVisible(false);
    }
//    ui->edit_IPV4Ip->setIP(setting.value("IPADDR").toString());

//    ui->edit_IPV4Mask->setIP(QHostAddress(len2netmask_str(setting.value("PREFIX").toInt())).toString());
//    ui->edit_IPV4Getway->setIP(setting.value("GATEWAY").toString());
//    ui->edit_IPV4DNS1->setIP(setting.value("DNS1").toString());
//    ui->edit_IPV4DNS2->setIP(setting.value("DNS2").toString());
//    QStringList list = setting.value("IPV6ADDR").toString().split("/");
//    if(!list.isEmpty())
//        ui->edit_IPV6Ip->setText(list.at(0));
//    else {
//        ui->edit_IPV6Ip->setText(setting.value("IPV6ADDR").toString());
//    }
//    if(list.size() == 2)
//        ui->edit_IPV6Mask->setText(list.at(1));
//    ui->edit_IPV6Getway->setText(setting.value("IPV6_DEFAULTGW").toString());
//    ui->edit_IPV6DNS1->setText(setting.value("DNS2").toString());
//    ui->edit_IPV6DNS2->setText(setting.value("DNS3").toString());
//    QScrollBar *pScrollBar = ui->scrollArea->verticalScrollBar();
//    if (pScrollBar)
//    {
//        int nMin = pScrollBar->minimum();
//        pScrollBar->setValue(nMin);
//    }
}

void WifiInfoWidget::applyBtnClicked()
{
    bool bSuccess = true;
    NetworkManager::WirelessDevice* device = nullptr;
    NetworkManager::WirelessNetwork::Ptr network;
    NetworkManager::Connection::Ptr conn;
    Common::ChattrDir chat;
    if(ui->comboBox_IPV4->currentIndex() == 1)
    {
        if(!QHostAddress().setAddress(ui->edit_IPV4Ip->getIP()))
        {
            ui->edit_IPV4Ip->setBorderWidth(1);
            bSuccess = false;
        }
        if(!QHostAddress().setAddress(ui->edit_IPV4Mask->getIP()))
        {
            ui->edit_IPV4Mask->setBorderWidth(1);
            bSuccess = false;
        }
        if(!ui->edit_IPV4Getway->getIP().isEmpty() && ui->edit_IPV4Getway->getIP() != "...")
        {
            if(!QHostAddress().setAddress(ui->edit_IPV4Getway->getIP()))
            {
                ui->edit_IPV4Getway->setBorderWidth(1);
                bSuccess = false;
            }
        }
    }
    if(!ui->edit_IPV4DNS1->getIP().isEmpty() && ui->edit_IPV4DNS1->getIP() != "...")
    {
        if(!QHostAddress().setAddress(ui->edit_IPV4DNS1->getIP()))
        {
            ui->edit_IPV4DNS1->setBorderWidth(1);
            bSuccess = false;
        }
    }
    if(!ui->edit_IPV4DNS2->getIP().isEmpty() && ui->edit_IPV4DNS2->getIP() != "...")
    {
        if(!QHostAddress().setAddress(ui->edit_IPV4DNS2->getIP()))
        {
            ui->edit_IPV4DNS2->setBorderWidth(1);
            bSuccess = false;
        }
    }
    if(ui->comboBox_IPV6->currentIndex() == 1)
    {
        if(QHostAddress(ui->edit_IPV6Ip->text()).protocol() != QAbstractSocket::IPv6Protocol)
        {
            ui->edit_IPV6Ip->setStyleSheet("border:1 solid red;background-color: rgb(255, 255, 255);");
            bSuccess = false;
        }
        if(ui->edit_IPV6Mask->text().toInt() <= 0)
        {
            ui->edit_IPV6Mask->setStyleSheet("border:1 solid red;background-color: rgb(255, 255, 255);");
            bSuccess = false;
        }
        if(!ui->edit_IPV6Getway->text().isEmpty())
        {
            if(QHostAddress(ui->edit_IPV6Getway->text()).protocol() != QAbstractSocket::IPv6Protocol)
            {
                ui->edit_IPV6Getway->setStyleSheet("border:1 solid red;background-color: rgb(255, 255, 255);");
                bSuccess = false;
            }
        }
    }
    if(!ui->edit_IPV6DNS1->text().isEmpty())
    {
        if(QHostAddress(ui->edit_IPV6DNS1->text()).protocol() != QAbstractSocket::IPv6Protocol)
        {
            ui->edit_IPV6DNS1->setStyleSheet("border:1 solid red;background-color: rgb(255, 255, 255);");
            bSuccess = false;
        }
    }
    if(!ui->edit_IPV6DNS2->text().isEmpty())
    {
        if(QHostAddress(ui->edit_IPV6DNS2->text()).protocol() != QAbstractSocket::IPv6Protocol)
        {
            ui->edit_IPV6DNS2->setStyleSheet("border:1 solid red;background-color: rgb(255, 255, 255);");
            bSuccess = false;
        }
    }
    if(!bSuccess) return;
    for (auto const & tmp : NetworkManager::networkInterfaces())
    {
        if(tmp->type() == NetworkManager::Device::Wifi && tmp->interfaceName() == m_device)
        {
            device = qobject_cast<NetworkManager::WirelessDevice *>(tmp.data());
            break;
        }
    }
    if(device == nullptr) return;
    network = device->findNetwork(m_ssid);
    if(network.isNull()) return;
    for (auto const & dev_conn : device->availableConnections())
    {
        if (dev_conn->settings()->id() == m_ssid)
        {
            conn = dev_conn;
        }
    }
    if(conn && conn->isValid())
    {
        NetworkManager::ConnectionSettings::Ptr settings = conn->settings();
        NetworkManager::WirelessSetting::Ptr wirelessSetting = settings->setting(NetworkManager::Setting::Wireless).dynamicCast<NetworkManager::WirelessSetting>();
        NetworkManager::WirelessSecuritySetting::Ptr wifiSecurity = settings->setting(NetworkManager::Setting::WirelessSecurity).dynamicCast<NetworkManager::WirelessSecuritySetting>();
        NetworkManager::Ipv4Setting::Ptr ipv4Setting = settings->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();
        NetworkManager::Ipv6Setting::Ptr ipv6Setting = settings->setting(NetworkManager::Setting::Ipv6).dynamicCast<NetworkManager::Ipv6Setting>();

        QList<NetworkManager::IpAddress> addrs = ipv4Setting->addresses();
        if(ui->comboBox_IPV4->currentIndex() == 0)
        {
            ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Automatic);
            ipv4Setting->setGateway("");
            addrs.clear();
        }
        else if(ui->comboBox_IPV4->currentIndex() == 2)
        {
            ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Disabled);
            ipv4Setting->setGateway("");
            ipv4Setting->setAddressData(NMVariantMapList());
            addrs.clear();
        }
        else {
            ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Manual);

            if(addrs.isEmpty())
            {
                if(!ui->edit_IPV4Ip->getIP().isEmpty())
                {
                    NetworkManager::IpAddress addr;
                    addr.setIp(QHostAddress(ui->edit_IPV4Ip->getIP()));
                    addr.setNetmask(QHostAddress(ui->edit_IPV4Mask->getIP()));
                    addr.setGateway(QHostAddress(ui->edit_IPV4Getway->getIP()));
                    addrs.push_back(addr);
                }
            }
            else {
                NetworkManager::IpAddress addr = addrs.first();
                addr.setIp(QHostAddress(ui->edit_IPV4Ip->getIP()));
                addr.setNetmask(QHostAddress(ui->edit_IPV4Mask->getIP()));
                addr.setGateway(QHostAddress(ui->edit_IPV4Getway->getIP()));
                addrs.replace(0, addr);
            }
        }
        ipv4Setting->setAddresses(addrs);
        QList<QHostAddress> ipv4_dns;
        //如该method是禁用,清空dns设置
        if(ui->comboBox_IPV4->currentIndex() != 2)
        {
            if(!ui->edit_IPV4DNS1->getIP().isEmpty() && ui->edit_IPV4DNS1->getIP() != "...")
                ipv4_dns.push_back(QHostAddress(ui->edit_IPV4DNS1->getIP()));
            if(!ui->edit_IPV4DNS2->getIP().isEmpty() && ui->edit_IPV4DNS2->getIP() != "...")
                ipv4_dns.push_back(QHostAddress(ui->edit_IPV4DNS2->getIP()));
        }
        ipv4Setting->setDns(ipv4_dns);

        QList<NetworkManager::IpAddress> addrs2 = ipv6Setting->addresses();
        if(ui->comboBox_IPV6->currentIndex() == 0)
        {
            ipv6Setting->setMethod(NetworkManager::Ipv6Setting::Automatic);
            addrs2.clear();
        }
        else if(ui->comboBox_IPV6->currentIndex() == 2)
        {
            ipv6Setting->setMethod(NetworkManager::Ipv6Setting::Ignored);
            ipv6Setting->setAddressData(NMVariantMapList());
            addrs2.clear();
        }
        else {
            ipv6Setting->setMethod(NetworkManager::Ipv6Setting::Manual);

            if(addrs2.isEmpty())
            {
                if(!ui->edit_IPV6Ip->text().isEmpty())
                {
                    NetworkManager::IpAddress addr;
                    addr.setIp(QHostAddress(ui->edit_IPV6Ip->text()));
                    addr.setGateway(QHostAddress(ui->edit_IPV6Getway->text()));
                    addr.setPrefixLength(ui->edit_IPV6Mask->text().toInt());
                    addrs2.push_back(addr);
                }
            }
            else {
                NetworkManager::IpAddress addr = addrs2.first();
                addr.setIp(QHostAddress(ui->edit_IPV6Ip->text()));
                addr.setPrefixLength(ui->edit_IPV6Mask->text().toInt());
                addr.setGateway(QHostAddress(ui->edit_IPV6Getway->text()));
                addrs2.replace(0, addr);
            }
        }
        ipv6Setting->setAddresses(addrs2);
        NetworkManager::IpRoute route;
        route.setIp(QHostAddress(ui->edit_IPV6Getway->text()));
        ipv6Setting->setRoutes(QList<NetworkManager::IpRoute>() << route);
        QList<QHostAddress> ipv6_dns;
        if(ui->comboBox_IPV6->currentIndex() != 2)
        {
            if(!ui->edit_IPV6DNS1->text().isEmpty())
                ipv6_dns.push_back(QHostAddress(ui->edit_IPV6DNS1->text()));
            if(!ui->edit_IPV6DNS2->text().isEmpty())
                ipv6_dns.push_back(QHostAddress(ui->edit_IPV6DNS2->text()));
        }
        ipv6Setting->setDns(ipv6_dns);

        QDBusPendingReply<> reply = conn->update(settings->toMap());
        reply.waitForFinished();
        if(!reply.isValid())
            qDebug() << "============" << reply.error();
        conn->save();

        QString command = QString::fromUtf8("sudo nmcli c reload");
        QProcess::execute(command);
        command = QString::fromUtf8("nmcli d reapply \"%1\"").arg(m_device);
        QProcess::execute(command);
    }
    else {
        creatNewConnection();
    }
    Q_EMIT backPage();
}

void WifiInfoWidget::backBtnClicked()
{
    clear();
}

QList<QPair<int, QString> > WifiInfoWidget::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_FORGET, tr("Forget Password"));
    pairs << QPair<int, QString>(Page::PAGE_BBTN_APPLY, g_tr_apply);
    return pairs;
}

QList<int> WifiInfoWidget::buttomBtnsVisible()
{
    QList<int> page_bbtn_keys;
    page_bbtn_keys << Page::PAGE_BBTN_APPLY;
    return page_bbtn_keys;
}

void WifiInfoWidget::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_FORGET)
    {
        Common::ChattrDir chat;
        //    m_item->onLinkForget();
        QString command = QString::fromUtf8("nmcli connect delete \"%1\"").arg(m_ssid);
        QProcess::execute(command);
        //删除保存的密钥
        NMConfig::global()->removeWiFIPasswd(m_ssid);
        Q_EMIT backPage();
    }
}

void WifiInfoWidget::refreshPage()
{
    NetworkManager::WirelessDevice* device = nullptr;
    for (auto const & tmp : NetworkManager::networkInterfaces())
    {
        //无线设备
        if(tmp->type() == NetworkManager::Device::Wifi && tmp->interfaceName() == m_device)
        {
            device = qobject_cast<NetworkManager::WirelessDevice *>(tmp.data());
            break;
        }
    }
    if(device == nullptr) return;

    NetworkManager::Connection::Ptr conn;
    for (auto const & dev_conn : device->availableConnections())
    {
        if (dev_conn->settings()->id() == m_ssid)
        {
            conn = dev_conn;
        }
    }
    if(conn && conn->isValid())
    {
        ui->widget_info->setVisible(true);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_FORGET, true);
    }
    else {
        ui->widget_info->setVisible(false);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_FORGET, false);
    }
}

void WifiInfoWidget::onIPV4MethodIndexChanged(int index)
{
    switch (index) {
    case 0:
        ui->widget_IPV4Ip->setVisible(false);
        ui->widget_IPV4Mask->setVisible(false);
        ui->widget_IPV4Getway->setVisible(false);
        ui->widget_IPV4DNS1->setVisible(true);
        ui->widget_IPV4DNS2->setVisible(true);
        break;
    case 1:
        ui->widget_IPV4Ip->setVisible(true);
        ui->widget_IPV4Mask->setVisible(true);
        ui->widget_IPV4Getway->setVisible(true);
        ui->widget_IPV4DNS1->setVisible(true);
        ui->widget_IPV4DNS2->setVisible(true);
    break;
    case 2:
        ui->widget_IPV4Ip->setVisible(false);
        ui->widget_IPV4Mask->setVisible(false);
        ui->widget_IPV4Getway->setVisible(false);
        ui->widget_IPV4DNS1->setVisible(false);
        ui->widget_IPV4DNS2->setVisible(false);
    break;
    }
}

void WifiInfoWidget::onIPV6MethodIndexChanged(int index)
{
    switch (index) {
    case 0:
        ui->widget_IPV6Ip->setVisible(false);
        ui->widget_IPV6Mask->setVisible(false);
        ui->widget_IPV6Getway->setVisible(false);
        ui->widget_IPV6DNS1->setVisible(true);
        ui->widget_IPV6DNS2->setVisible(true);
        break;
    case 1:
        ui->widget_IPV6Ip->setVisible(true);
        ui->widget_IPV6Mask->setVisible(true);
        ui->widget_IPV6Getway->setVisible(true);
        ui->widget_IPV6DNS1->setVisible(true);
        ui->widget_IPV6DNS2->setVisible(true);
    break;
    case 2:
        ui->widget_IPV6Ip->setVisible(false);
        ui->widget_IPV6Mask->setVisible(false);
        ui->widget_IPV6Getway->setVisible(false);
        ui->widget_IPV6DNS1->setVisible(false);
        ui->widget_IPV6DNS2->setVisible(false);
    break;
    }
}

void WifiInfoWidget::onIPV6Textchanged(const QString &text)
{
    if(text.length() > 0)
    {
        QLineEdit* edit = static_cast<QLineEdit*>(sender());
        edit->setStyleSheet("border:0 solid red;background-color: rgb(255, 255, 255);");
    }
}

int WifiInfoWidget::netmask_str2len(QString mask)
{
    int netmask = 0;
    unsigned int mask_tmp;

    mask_tmp = ntohl((int)inet_addr(mask.toLocal8Bit().data()));
    while (mask_tmp & 0x80000000)
    {
        netmask++;
        mask_tmp = (mask_tmp << 1);
    }

    return netmask;
}

QString WifiInfoWidget::len2netmask_str(int len)
{
    unsigned int addr;
    char mask[32];
    unsigned char n = len; //传过来的掩码位数
    addr = LMOVE(0xffffffff,32-n);//u32表示掩码(中间变量)
    sprintf(mask,"%u.%u.%u.%u",RMOVE(addr,24),RMOVE(addr,16)&0xff,RMOVE(addr,8)&0xff,addr&0xff);//点分十进制表示掩码
    return QString(mask);
}

void WifiInfoWidget::creatNewConnection()
{
    Common::ChattrDir chat;
    QString command = QString::fromUtf8("nmcli connect delete \"%1\"").arg(m_ssid);
    QProcess::execute(command);
    QString method = "auto";
    if(ui->comboBox_IPV4->currentIndex() == 0)
    {
        method = "auto";
    }
    else {
        method = "manual";
    }
    command = QString::fromUtf8("nmcli connection add con-name %1 ifname %2 autoconnect yes type wifi ssid %3 ipv4.method %4 ")
            .arg(m_ssid).arg(m_device).arg(m_ssid).arg(method);
    QString ip = ui->edit_IPV4Ip->getIP();
    if(!ip.isEmpty() && ip != "...")
    {
        command += QString(" ip4 " + ip);
        if(!ui->edit_IPV4Mask->getIP().isEmpty())
        {
            command += QString("/%1").arg(netmask_str2len(ui->edit_IPV4Mask->getIP()));
        }
    }
    QString gateway = ui->edit_IPV4Getway->getIP();
    if(!gateway.isEmpty() && gateway != "...")
    {
        command += QString(" gw4 " + gateway);
    }

    QString dns = "";
    if(!ui->edit_IPV4DNS1->getIP().isEmpty() && ui->edit_IPV4DNS1->getIP() != "...")
    {
        dns += QString(" ipv4.dns " + ui->edit_IPV4DNS1->getIP());
        if(!ui->edit_IPV4DNS2->getIP().isEmpty() && ui->edit_IPV4DNS2->getIP() != "...")
            dns += QString("," + ui->edit_IPV4DNS2->getIP());
    }
    if(!dns.isEmpty())
    {
        command += QString(dns);
    }
    command += QString(" wifi-sec.key-mgmt wpa-psk");
    qDebug() << command;
    QProcess::execute(command);
    return;
    qDBusRegisterMetaType<NMVariantMapMap>();
    QTextStream qout(stdout, QIODevice::WriteOnly);
    QTextStream qin(stdin, QIODevice::ReadOnly);

    NetworkManager::ConnectionSettings *settings = new NetworkManager::ConnectionSettings(NetworkManager::ConnectionSettings::Wireless);
    const NetworkManager::Device::List deviceList = NetworkManager::networkInterfaces();

    NetworkManager::WirelessDevice::Ptr wifiDevice;

    // We have to find some wireless device
    for (NetworkManager::Device::Ptr dev : deviceList) {
        if (dev->type() == NetworkManager::Device::Wifi && dev->interfaceName() == m_device) {
            wifiDevice = qobject_cast<NetworkManager::WirelessDevice *>(dev);
            break;
        }
    }

    if (!wifiDevice) {
        return;
    }

    const QStringList accessPointList = wifiDevice->accessPoints();
    QString ssid;
    QString result;
    QString accessPointPath;

    // Check for available accesspoint
    for (const QString &ap : accessPointList) {
        NetworkManager::AccessPoint accessPoint(ap);
        // For simplification we use APs only with Wep security or without any security
        /* clang-format off */
        if (m_ssid == accessPoint.ssid()) { /* clang-format on */
            ssid = accessPoint.ssid();
            accessPointPath = ap;
            break;
        }
    }

    // Now we will prepare our new connection, we have to specify ID and create new UUID
    settings->setId(ssid);
    settings->setUuid(QUuid::createUuid().toString().mid(1, QUuid::createUuid().toString().length() - 2));

    // For wireless setting we have to specify SSID
    NetworkManager::WirelessSetting::Ptr wirelessSetting = settings->setting(NetworkManager::Setting::Wireless).dynamicCast<NetworkManager::WirelessSetting>();
    wirelessSetting->setSsid(ssid.toUtf8());

    NetworkManager::Ipv4Setting::Ptr ipv4Setting = settings->setting(NetworkManager::Setting::Ipv4).dynamicCast<NetworkManager::Ipv4Setting>();
    QList<NetworkManager::IpAddress> addrs = ipv4Setting->addresses();
    if(ui->comboBox_IPV4->currentIndex() == 0)
    {
        ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Automatic);
        addrs.clear();
    }
    else {
        ipv4Setting->setMethod(NetworkManager::Ipv4Setting::Manual);

        if(addrs.isEmpty())
        {
            if(!ui->edit_IPV4Ip->getIP().isEmpty())
            {
                NetworkManager::IpAddress addr;
                addr.setIp(QHostAddress(ui->edit_IPV4Ip->getIP()));
                addr.setNetmask(QHostAddress(ui->edit_IPV4Mask->getIP()));
                addrs.push_back(addr);
            }
        }
        else {
            NetworkManager::IpAddress addr = addrs.first();
            addr.setIp(QHostAddress(ui->edit_IPV4Ip->getIP()));
            addr.setNetmask(QHostAddress(ui->edit_IPV4Mask->getIP()));
            addrs.replace(0, addr);
        }
    }
    ipv4Setting->setAddresses(addrs);
    ipv4Setting->setGateway(ui->edit_IPV4Getway->getIP());
    QList<QHostAddress> ipv4_dns;
    if(!ui->edit_IPV4DNS1->getIP().isEmpty())
        ipv4_dns.push_back(QHostAddress(ui->edit_IPV4DNS1->getIP()));
    if(!ui->edit_IPV4DNS2->getIP().isEmpty())
        ipv4_dns.push_back(QHostAddress(ui->edit_IPV4DNS2->getIP()));
    ipv4Setting->setDns(ipv4_dns);

    // Optional password setting. Can be skipped if you do not need encryption.
//    NetworkManager::WirelessSecuritySetting::Ptr wifiSecurity =
//        settings->setting(NetworkManager::Setting::WirelessSecurity).dynamicCast<NetworkManager::WirelessSecuritySetting>();
//    wifiSecurity->setKeyMgmt(NetworkManager::WirelessSecuritySetting::WpaPsk);
//    wifiSecurity->setPsk("12345678");
//    wifiSecurity->setInitialized(true);
//    wirelessSetting->setSecurity("802-11-wireless-security");

    // We try to add and activate our new wireless connection
    QDBusPendingReply<QDBusObjectPath, QDBusObjectPath> reply = NetworkManager::addAndActivateConnection(settings->toMap(), wifiDevice->uni(), accessPointPath);

    reply.waitForFinished();

    // Check if this connection was added successfuly
    if (reply.isValid()) {
        // Now our connection should be added in NetworkManager and we can print all settings pre-filled from NetworkManager
        NetworkManager::Connection connection(reply.value().path());
        NetworkManager::ConnectionSettings::Ptr newSettings = connection.settings();
        // Print resulting settings
        qDebug() << (*newSettings.data());

        // Continue with adding secrets
        NetworkManager::WirelessSecuritySetting::Ptr wirelessSecuritySetting =
            newSettings->setting(NetworkManager::Setting::WirelessSecurity).dynamicCast<NetworkManager::WirelessSecuritySetting>();
        if (!wirelessSecuritySetting->needSecrets().isEmpty()) {
            qDebug() << "Need secrets: " << wirelessSecuritySetting->needSecrets();
            // TODO: fill missing secrets
        }

    } else {
        qDebug() << "Connection failed: " << reply.error();
    }
}

void WifiInfoWidget::clear()
{
    ui->label_ssid->setText(tr("None"));
    ui->label_protocol->setText(tr("None"));
    ui->label_securityType->setText(tr("None"));
    ui->label_netRate->setText(tr("None"));
    ui->label_IPV4Addr1->setText(tr("None"));
    ui->label_netChannel->setText(tr("None"));
    ui->label_IPV4Addr2->setText(tr("None"));
    ui->label_IPV6Addr->setText(tr("None"));
    ui->label_DNS->setText(tr("None"));
    ui->label_mac->setText(tr("None"));
    ui->label_speed->setText(tr("None"));
    ui->comboBox_IPV4->setCurrentIndex(0);
    onIPV4MethodIndexChanged(0);
    ui->comboBox_IPV6->setCurrentIndex(0);
    onIPV6MethodIndexChanged(0);
    ui->edit_IPV4Ip->clear();
    ui->edit_IPV4Mask->clear();
    ui->edit_IPV4Getway->clear();
    ui->edit_IPV4DNS1->clear();
    ui->edit_IPV4DNS2->clear();
    ui->edit_IPV6Ip->clear();
    ui->edit_IPV6Mask->clear();
    ui->edit_IPV6Getway->clear();
    ui->edit_IPV6DNS1->clear();
    ui->edit_IPV6DNS2->clear();

    ui->widget_ipv4->setVisible(false);
    ui->frame_5->setVisible(false);
}
