#ifndef WIFILISTWIDGETITEM_H
#define WIFILISTWIDGETITEM_H

#include <QWidget>
#include <nde/Nde/Settings>
#include <NetworkManagerQt/WirelessNetwork>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include <NetworkManagerQt/WirelessSetting>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/Ipv4Setting>
#include <NetworkManagerQt/Ipv6Setting>
#include <NetworkManagerQt/Utils>

namespace Ui {
class WifiListWidgetItem;
}

class WifiListWidgetItem : public QWidget
{
    Q_OBJECT
    enum ENUM_ShowType{
        ShowInit,
        ShowStatus,    //default
        ShowLink,
        ShowInputKey,
        ShowLinking,
        ShowLinkFinished
    };
public:
    explicit WifiListWidgetItem(NetworkManager::WirelessDevice * device, NetworkManager::WirelessNetwork::Ptr network, QWidget *parent = nullptr);
    ~WifiListWidgetItem();
    NetworkManager::WirelessNetwork::Ptr getNetwork() const;
    void setNetwork(NetworkManager::WirelessNetwork::Ptr network){m_network = network;};
    NetworkManager::WirelessDevice* getDevice() const;
    QString getSecType() const;
    //获取该wifi建立的连接
    NetworkManager::Connection::Ptr getConnect();
    //检查是否是已知
    bool checkKnown();

    //更新该连接的激活状态
    void updateActiveStatu();
    bool getActiveStatu();

    void setEditVisible(bool b);
    bool getEditVisible();
    void setEditFocus(bool b);
    bool getEditFocus();
    void setToDelete(){m_bToDelete = true;}
    std::pair<QString, int> getEditText();
    void setEditText(std::pair<QString, int> text);
    bool getCheckBoxStatu();
    void setCheckBoxStatu(bool b);

Q_SIGNALS:
    void itemClicked();
    void editClicked();
    void itemDelete();
public Q_SLOTS:
    void onSignalStrengthChanged(int signalStrength);
    void onLinkForget();
    void onLinkBreak();
    void onBtnCancelClicked();
    void onBtnConnectClicked();
    void onBtnDisconnectClicked();
    void onPasswordTextChanged();

protected:
    void mousePressEvent(QMouseEvent *event) override;
    void mouseReleaseEvent(QMouseEvent *event) override;
    void paintEvent(QPaintEvent *event) override;
    bool eventFilter(QObject *watched, QEvent *event) override;

private:
    void showCurrentLinkStatus(ENUM_ShowType type);
    void showStatus();
    void showLink();
    void showInputKey();
    void showLinking();
    void showFinished();
    void connectByPwd(const QString pwd);
    void connectNoPwd();

    QString getIconBySignalStrength(int strength);
    //检查是否激活
    NetworkManager::ActiveConnection::State checkActive();
    //检查是否是自动连接
    bool checkAuto();
    //检查该wifi是否需要密码，返回true：password为密码，返回false：password为空
    bool checkNeedPassword(QString & password);

private:
    Ui::WifiListWidgetItem *ui;
    NetworkManager::WirelessDevice* m_device;
    NetworkManager::WirelessNetwork::Ptr m_network;
    QString m_ssid;
    int m_signalStrength;
    ENUM_ShowType m_showType;
    NetworkManager::ActiveConnection::State m_active;  //激活状态
    bool m_known;   //记住
    bool m_auto;    //自动连接
    NetworkManager::WirelessSecurityType m_secType;
    bool m_needPassword; //该wifi是否需要密码
    int mConnectingTimerCount; //动画计数
    QTimer *mConnectingTimer; //连接中动画
    bool m_bClicked;
    bool m_bToDelete;
    QList<QVariant> m_failMsgs;
};

#endif // WIFILISTWIDGETITEM_H
