#include "wlan.h"
#include "ui_wlan.h"
#include <QProcess>
#include <qalgorithms.h>
#include "NetworkManagerQt/WirelessSetting"
#include "wifiinfowidget.h"
#include "newwifiwidget.h"
#include <QScrollBar>
#include <QFile>

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_ADD (Page::PAGE_BBTN_USER+1)

WLan::WLan(QWidget *parent) :
    Page(parent),
    ui(new Ui::WLan),
    m_toDelItem(nullptr)
{
    ui->setupUi(this);
    QFile f(":/qss/wlan.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    connect(NetworkManager::notifier(), &NetworkManager::Notifier::statusChanged, this, &WLan::onActiveConnectionsChanged);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::wirelessEnabledChanged, this, &WLan::onWirelessEnabledChanged);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::deviceAdded, this, &WLan::onDeviceAdded);
    connect(NetworkManager::notifier(), &NetworkManager::Notifier::deviceRemoved, this, &WLan::onDeviceAdded);

    m_wifiStatus = NetworkManager::isWirelessEnabled();
    initUI();
    for (auto const & device : NetworkManager::networkInterfaces())
    {
        //无线设备
        if(device->type() == NetworkManager::Device::Wifi)
        {
            addDevice(device);
            break;
        }
    }

    m_freshTimer = new QTimer(this);
    connect(m_freshTimer, &QTimer::timeout, this, &WLan::onFreshTimeout);
    m_freshTimer->setInterval(5 * 1000);
    m_freshTimer->start();
    onFreshTimeout();

    m_freshUITimer = new QTimer(this);
    connect(m_freshUITimer, &QTimer::timeout, this, &WLan::onFreshUITimeout);
    m_freshUITimer->setInterval(5 * 1000);

    sortList(SortByName);
    onActiveConnectionsChanged();
}

WLan::~WLan()
{
    delete ui;
}

QList<QPair<int, QString> > WLan::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_ADD, tr("Add"));
    return pairs;
}

QList<int> WLan::buttomBtnsVisible()
{
    QList<int> page_bbtn_keys;
    page_bbtn_keys << PAGE_BUTTOM_BTN_ADD;
    return page_bbtn_keys;
}

void WLan::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_ADD)
    {
        QString device = m_devices.isEmpty() ? "" : m_devices.at(0)->interfaceName();
        Q_EMIT signalAddWifi(device);
    }
}

void WLan::refreshPage()
{

}

void WLan::onWifiSwitchToggle(bool checked)
{
    NetworkManager::setWirelessEnabled(checked);
}

void WLan::onFreshTimeout()
{
    onRefreshWifiList();
}

void WLan::onFreshUITimeout()
{
    //连接wifi时网卡驱动会把所有wifi删除再添加,删除操作延迟刷新UI
    if (m_freshUITimer->isActive())
        m_freshUITimer->stop();
    m_toDelItem = nullptr;
    QList<WifiListWidgetItem*>::iterator it = m_itemWidgets.begin();
    for (; it != m_itemWidgets.end(); it++)
    {
        bool bExit = false;
        for (QList<WifiListWidgetItem*>::iterator it1 = m_itemWidgets_tmp.begin(); it1 != m_itemWidgets_tmp.end(); it1++)
        {
            if((*it) == (*it1))
            {
                bExit = true;
                break;
            }
        }
        if(!bExit)
        {
            //如果正在输入密码，保留item
            if((*it)->getEditVisible() && (*it)->getEditFocus())
            {
                m_toDelItem = (*it);
                m_toDelItem->setToDelete();
                continue;
            }
            (*it)->deleteLater();
        }
    }
    m_itemWidgets = m_itemWidgets_tmp;
    if(m_toDelItem)
    {
        m_itemWidgets.push_back(m_toDelItem);
    }
    sortList(SortByName);
    update();
}

void WLan::onItemClicked()
{
    WifiListWidgetItem* item = static_cast<WifiListWidgetItem*>(sender());
    for(auto& tmp : m_itemWidgets)
    {
        if(tmp != item)
        {
            tmp->setEditVisible(false);
        }
    }
}

void WLan::onItemDelete()
{
//    qDebug() << __FUNCTION__ << __LINE__;
    WifiListWidgetItem* obj = (WifiListWidgetItem*)sender();
    if(obj && !m_toDelItem) m_toDelItem = obj;
    if(!m_toDelItem) return;
    ui->wifiListLayout->removeWidget(m_toDelItem);
    m_itemWidgets.removeOne(m_toDelItem);
    m_itemWidgets_tmp.removeOne(m_toDelItem);
    ui->wifiListLayout->update();
    m_toDelItem->deleteLater();
    m_toDelItem = nullptr;
}

void WLan::onItemEditClicked()
{
    WifiListWidgetItem* item = static_cast<WifiListWidgetItem*>(sender());
    Q_EMIT signalEditWifi(item->getDevice()->interfaceName(), item->getNetwork()->ssid());
}

void WLan::onAddNewClicked()
{

}

void WLan::onRefreshWifiList()
{
    for(auto d : m_devices)
    {
        NetworkManager::WirelessDevice*  wd = qobject_cast<NetworkManager::WirelessDevice*>(d.data());
        if(wd)
        {
            QDBusPendingReply<> reply = wd->requestScan();
            QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, wd);
            connect(watcher, &QDBusPendingCallWatcher::finished, [wd] (QDBusPendingCallWatcher * watcher) {
                if (watcher->isError() || !watcher->isValid())
                {
                    //TODO: in what form should we output the warning messages
                    qWarning() << QStringLiteral("requestScan on device '%1' failed: %3").arg(wd->interfaceName())
                                  .arg(watcher->error().message());
                }
                watcher->deleteLater();
            });
        }
    }
}

void WLan::onBitRateChanged(int bitrate)
{

}

void WLan::onActiveConnectionsChanged()
{
    for(WifiListWidgetItem * item : m_itemWidgets)
    {
        if(item->getActiveStatu())
        {
            if(ui->wifiListLayout->indexOf(item) != 0)
            {
                ui->wifiListLayout->removeWidget(item);
                ui->wifiListLayout->insertWidget(0, item);
                ui->wifiListLayout->update();
            }
        }
    }
}

void WLan::onWirelessEnabledChanged(bool status)
{
    m_wifiStatus = status;
    m_toggleWifiSwitch->setChecked(status);

    if(!m_wifiStatus)
    {
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_ADD, false);
        ui->verticalSpacer_3->changeSize(20, 40, QSizePolicy::Expanding, QSizePolicy::Expanding);
        ui->verticalLayout->update();
    }
    else {
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_ADD, true);
        ui->verticalSpacer_3->changeSize(20, 40, QSizePolicy::Expanding, QSizePolicy::Expanding);
        ui->verticalLayout->update();
    }
    update();
}

void WLan::onDeviceAdded(const QString &uni)
{
    auto device = NetworkManager::findNetworkInterface(uni);
    if(device->type() == NetworkManager::Device::Wifi)
    {
        addDevice(device);
    }
}

void WLan::onDeviceRemoved(const QString &uni)
{
    auto device = NetworkManager::findNetworkInterface(uni);
    if(device->type() == NetworkManager::Device::Wifi)
    {
        removeDevice(device);
    }
}

void WLan::onConnectionAdded(const QString &path)
{

}

void WLan::onConnectionRemoved(const QString &path)
{

}

void WLan::onDeviceUpdated()
{

}

void WLan::onActiveAccessPointChanged(const QString &access)
{

}

void WLan::onModeChanged(NetworkManager::WirelessDevice::OperationMode mode)
{

}

void WLan::onWirelessCapabilitiesChanged(NetworkManager::WirelessDevice::Capabilities cap)
{

}

void WLan::onHardwareAddressChanged(const QString &addr)
{

}

void WLan::onPermanentHardwareAddressChanged(const QString &addr)
{

}

void WLan::onWirelessPropertiesChanged(uint property)
{

}

void WLan::onAccessPointAppeared(const QString &uni)
{

}

void WLan::onAccessPointDisappeared(const QString &uni)
{

}

void WLan::onWifiNetworkAppeared(const QString &ssid)
{
    for(NetworkManager::Device::Ptr device : m_devices)
    {
        NetworkManager::WirelessDevice * wirelessDevice = qobject_cast<NetworkManager::WirelessDevice *>(device.data());
        if(wirelessDevice)
        {
            NetworkManager::WirelessNetwork::Ptr network = wirelessDevice->findNetwork(ssid);
            if(!network.isNull())
            {
//                qWarning() << __FUNCTION__ << __LINE__ << network->ssid();
                WifiListWidgetItem *netWidget = new WifiListWidgetItem(wirelessDevice, network);
                connect(network.data(), &NetworkManager::WirelessNetwork::signalStrengthChanged, netWidget, &WifiListWidgetItem::onSignalStrengthChanged);
                connect(netWidget, &WifiListWidgetItem::itemClicked, this, &WLan::onItemClicked);
                connect(netWidget, &WifiListWidgetItem::itemDelete, this, &WLan::onItemDelete);
                connect(netWidget, &WifiListWidgetItem::editClicked, this, &WLan::onItemEditClicked);
                int index = -1;
                WifiListWidgetItem* item_tmp = nullptr;
                for(int i = 0; i < m_itemWidgets.size(); i++)
                {
                    if(network->ssid() == m_itemWidgets[i]->getNetwork()->ssid()) item_tmp = m_itemWidgets[i];
                    if(!m_itemWidgets[i]->getActiveStatu() && QString::compare(network->ssid().toLower(), m_itemWidgets[i]->getNetwork()->ssid().toLower()) <= 0)
                        index = i;
                }
                if(index == -1)
                {
                    ui->wifiListLayout->insertWidget(ui->wifiListLayout->count(), netWidget);
                    m_itemWidgets.push_front(netWidget);
                }
                else {
                    m_itemWidgets.insert(index + 1, netWidget);
                    ui->wifiListLayout->insertWidget(m_itemWidgets.count() - index - 2, netWidget);
                }
                m_itemWidgets_tmp.push_front(netWidget);
                if(item_tmp !=  nullptr)
                {
                    //如果WiFi列表刷新前重新扫描到该WiFi，删除之前的item
                    if(m_toDelItem && m_toDelItem == item_tmp) {
                        m_toDelItem = nullptr;
                        qDebug() << __FUNCTION__ << __LINE__;
                        //还原输入中的状态
                        netWidget->setEditText(item_tmp->getEditText());
                        netWidget->setCheckBoxStatu(item_tmp->getCheckBoxStatu());
                        netWidget->setEditVisible(true);
                    }
                    ui->wifiListLayout->removeWidget(item_tmp);
                    m_itemWidgets.removeOne(item_tmp);
                    item_tmp->deleteLater();
                    item_tmp = nullptr;
                }
                break;
            }
        }
    }
    update();
}

void WLan::onWifiNetworkDisappeared(const QString &ssid)
{
    for(WifiListWidgetItem * item : m_itemWidgets_tmp)
    {
        if(item->getNetwork()->ssid() == ssid)
        {
            qWarning() << "WIFI删除连接" << item->getNetwork()->ssid();
            item->setToDelete();
            m_itemWidgets_tmp.removeAll(item);
            break;
        }
    }
    if (m_freshUITimer->isActive())
    {
        m_freshUITimer->stop();
    }
    m_freshUITimer->start();
    update();
}

void WLan::initUI()
{
    QHBoxLayout * displayCommonLayout = new QHBoxLayout();
    displayCommonLayout->setContentsMargins(0,0,0,0);
    displayCommonLayout->setSpacing(0);
    ui->wifiSwitchWidget->setLayout(displayCommonLayout);
    m_toggleWifiSwitch = new NdeWidget::QMaterialToggle(ui->wifiSwitchWidget);
    m_toggleWifiSwitch->setChecked(m_wifiStatus);
    connect(m_toggleWifiSwitch, &NdeWidget::QMaterialToggle::toggled, this, &WLan::onWifiSwitchToggle);
    displayCommonLayout->addWidget(m_toggleWifiSwitch);

    clearWifiList();
    onWirelessEnabledChanged(m_wifiStatus);
}

void WLan::addDevice(NetworkManager::Device::Ptr device)
{
    m_devices.append(device);
    connect(device.data(), &NetworkManager::Device::stateChanged, this, &WLan::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::activeConnectionChanged, this, &WLan::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::autoconnectChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::availableConnectionChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::availableConnectionAppeared, this, &WLan::onConnectionAdded);
    connect(device.data(), &NetworkManager::Device::availableConnectionDisappeared, this, &WLan::onConnectionRemoved);
    connect(device.data(), &NetworkManager::Device::capabilitiesChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::dhcp4ConfigChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::dhcp6ConfigChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::driverChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::driverVersionChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::firmwareMissingChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::firmwareVersionChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::interfaceNameChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipV4AddressChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipV4ConfigChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipV6ConfigChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::ipInterfaceChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::managedChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::physicalPortIdChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::mtuChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::nmPluginMissingChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::meteredChanged, this, &WLan::onDeviceUpdated);
    connect(device.data(), &NetworkManager::Device::connectionStateChanged, this, &WLan::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::stateReasonChanged, this, &WLan::onActiveConnectionsChanged);
    connect(device.data(), &NetworkManager::Device::udiChanged, this, &WLan::onDeviceUpdated);
    NetworkManager::WirelessDevice * wirelessDevice = qobject_cast<NetworkManager::WirelessDevice *>(device.data());
    if(wirelessDevice)
    {
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::bitRateChanged, this, &WLan::onBitRateChanged);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::activeAccessPointChanged, this, &WLan::onActiveAccessPointChanged);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::modeChanged, this, &WLan::onModeChanged);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::wirelessCapabilitiesChanged, this, &WLan::onWirelessCapabilitiesChanged);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::hardwareAddressChanged, this, &WLan::onHardwareAddressChanged);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::permanentHardwareAddressChanged, this, &WLan::onPermanentHardwareAddressChanged);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::wirelessPropertiesChanged, this, &WLan::onWirelessPropertiesChanged);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::accessPointAppeared, this, &WLan::onAccessPointAppeared);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::accessPointDisappeared, this, &WLan::onAccessPointDisappeared);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::networkAppeared, this, &WLan::onWifiNetworkAppeared);
        disconnect(wirelessDevice, &NetworkManager::WirelessDevice::networkDisappeared, this, &WLan::onWifiNetworkDisappeared);

        connect(wirelessDevice, &NetworkManager::WirelessDevice::bitRateChanged, this, &WLan::onBitRateChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::activeAccessPointChanged, this, &WLan::onActiveAccessPointChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::modeChanged, this, &WLan::onModeChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::wirelessCapabilitiesChanged, this, &WLan::onWirelessCapabilitiesChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::hardwareAddressChanged, this, &WLan::onHardwareAddressChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::permanentHardwareAddressChanged, this, &WLan::onPermanentHardwareAddressChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::wirelessPropertiesChanged, this, &WLan::onWirelessPropertiesChanged);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::accessPointAppeared, this, &WLan::onAccessPointAppeared);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::accessPointDisappeared, this, &WLan::onAccessPointDisappeared);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::networkAppeared, this, &WLan::onWifiNetworkAppeared);
        connect(wirelessDevice, &NetworkManager::WirelessDevice::networkDisappeared, this, &WLan::onWifiNetworkDisappeared);

        for(auto const & network : wirelessDevice->networks())
        {
            WifiListWidgetItem *netWidget = new WifiListWidgetItem(wirelessDevice, network);
//            qWarning() << __FUNCTION__ << __LINE__ << network->ssid() << network->accessPoints().at(0)->ssid();
            connect(network.data(), &NetworkManager::WirelessNetwork::signalStrengthChanged, netWidget, &WifiListWidgetItem::onSignalStrengthChanged);
            connect(netWidget, &WifiListWidgetItem::itemClicked, this, &WLan::onItemClicked);
            connect(netWidget, &WifiListWidgetItem::itemDelete, this, &WLan::onItemDelete);
            connect(netWidget, &WifiListWidgetItem::editClicked, this, &WLan::onItemEditClicked);

            ui->wifiListLayout->insertWidget(ui->wifiListLayout->count(), netWidget);
            m_itemWidgets.append(netWidget);
        }
    }
    m_itemWidgets_tmp = m_itemWidgets;
}

void WLan::removeDevice(NetworkManager::Device::Ptr device)
{
    if(m_devices.contains(device))
    {
        m_devices.removeAll(device);
    }
}

void WLan::sortList(SortRole role)
{
    switch (role) {
    case SortByName:
        qSort(m_itemWidgets.begin(), m_itemWidgets.end(), [](WifiListWidgetItem*a, WifiListWidgetItem* b)->bool{
            if(a->getActiveStatu()) return false;
            else if(b->getActiveStatu()) return true;
            else
            {
                int ret = QString::compare(a->getNetwork()->ssid().toLower(), b->getNetwork()->ssid().toLower());
                return ret <= 0 ? false : true;
            }
        });
        break;
    case SortBySignal:
        qSort(m_itemWidgets.begin(), m_itemWidgets.end(), [](WifiListWidgetItem*a, WifiListWidgetItem* b)->bool{
            if(a->getActiveStatu()) return false;
            else if(b->getActiveStatu()) return true;
            else return a->getNetwork()->signalStrength() < b->getNetwork()->signalStrength();
        });
        break;
    }
    QList<WifiListWidgetItem*>::iterator it = m_itemWidgets.begin();
    for(; it != m_itemWidgets.end(); it++)
    {
        ui->wifiListLayout->removeWidget(*it);
        ui->wifiListLayout->insertWidget(0, *it);
    }
    ui->wifiListLayout->update();
}

void WLan::clearWifiList()
{
    QList<WifiListWidgetItem*>::iterator it = m_itemWidgets.begin();
    for(; it != m_itemWidgets.end(); it++)
    {
        ui->wifiListLayout->removeWidget(*it);
        delete *it;
        *it = nullptr;
    }
    m_itemWidgets.clear();
    m_itemWidgets_tmp.clear();
}

bool WLan::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type())
    {
    case QEvent::MouseButtonRelease:
        if(watched == this)
        {
            for(auto& item : m_itemWidgets)
            {
                //隐藏输入框
                item->setEditVisible(false);
            }
            onItemDelete();
        }
        break;
    }
    return QWidget::eventFilter(watched, event);
}
