#ifndef WLAN_H
#define WLAN_H

#include <QWidget>
#include <designer/ndewidgetmaterialtoggle.h>
#include <NetworkManagerQt/WirelessDevice>
#include <NetworkManagerQt/Device>
#include <NetworkManagerQt/ConnectionSettings>
#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/Settings>
#include "wifilistwidgetitem.h"
#include "../baseui/page.h"

class WifiInfoWidget;
class NewWifiWidget;

namespace Ui {
class WLan;
}

class WLan : public Page
{
    Q_OBJECT

    typedef  enum SortRole
    {
        SortByName,
        SortBySignal
    }SortRole;

public:
    explicit WLan(QWidget *parent = nullptr);
    ~WLan();

public:
    //页面唯一键
    static QString pageKey() {return "wifi";}

    //页面标题
    virtual QString pageTitle() {return tr("WIFI");}

    //该页面底部的按钮QPair<key, text>
    virtual QList<QPair<int, QString> > buttomBtnsPair();
    virtual QList<int> buttomBtnsVisible();
    //底部按钮按下事件响应的接口
    virtual void buttomBtnClicked(const int key);

    //每次进入该界面需要做的操作
    virtual void refreshPage();

Q_SIGNALS:
    void signalAddWifi(const QString deviceName);
    void signalEditWifi(QString uni, QString ssid);

private Q_SLOTS:
    void onWifiSwitchToggle(bool checked);
    void onFreshTimeout();
    void onFreshUITimeout();
    void onItemClicked();
    void onItemDelete();
    void onItemEditClicked();
    void onAddNewClicked();
    void onRefreshWifiList();
    //关联NetWorkManager 信号的槽
    void onActiveConnectionsChanged();
    void onWirelessEnabledChanged(bool status);
    void onDeviceAdded(const QString &uni);
    void onDeviceRemoved(const QString &uni);
    //有线连接
    void onConnectionAdded(const QString &path);
    void onConnectionRemoved(const QString &path);

    //关联WIFI Device信号的槽
    void onDeviceUpdated();
    void onBitRateChanged(int bitrate);
    void onActiveAccessPointChanged(const QString &access);
    void onModeChanged(NetworkManager::WirelessDevice::OperationMode mode);
    void onWirelessCapabilitiesChanged(NetworkManager::WirelessDevice::Capabilities cap);
    void onHardwareAddressChanged(const QString &addr);
    void onPermanentHardwareAddressChanged(const QString &addr);
    void onWirelessPropertiesChanged(uint property);
    void onAccessPointAppeared(const QString &uni);
    void onAccessPointDisappeared(const QString &uni);
    void onWifiNetworkAppeared(QString const & ssid);
    void onWifiNetworkDisappeared(QString const & ssid);
private:
    void initUI();
    void addDevice(NetworkManager::Device::Ptr device);
    void removeDevice(NetworkManager::Device::Ptr device);
    void sortList(SortRole role);
    void clearWifiList();

private:
    Ui::WLan *ui;
    NdeWidget::QMaterialToggle * m_toggleWifiSwitch;
    bool m_wifiStatus;
    //无线设备列表
    NetworkManager::Device::List m_devices;
    QList<WifiListWidgetItem*> m_itemWidgets; //UI显示的Item
    QList<WifiListWidgetItem*> m_itemWidgets_tmp; //实际搜索的Item
    QTimer* m_freshTimer;
    QTimer* m_freshUITimer; //UI刷新缓冲定时器
    WifiListWidgetItem* m_toDelItem; //已经消失但还在输入密码的Item

protected:
    virtual bool eventFilter(QObject *watched, QEvent *event);
};

#endif // WLAN_H
